<?php

namespace Teknisa\OPL\Factories;

use Doctrine\ORM\EntityManager;

use Teknisa\Libs\Util\DateUtil;
use Geral\Entities\Itemproduto;
use Fiscal\Entities\Trabitproimpos;
use Fiscal\Entities\Trabimposnftot;
use Fiscal\Entities\Contapagar;
use Suprimentos\Entities\Nfrelacaf;

use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Dfs11000NfSaidaFactory {

    private $entityManager;

    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }

    public function quantidadeProgramadaAF($filial, $lancamento, $vrUnitario, $produto) {
        try {
             $sql = <<<SQL
             SELECT N.CDFILDEST, N.CDFILLANC, N.NRAUTOFORN, N.CDMARCA,
                   N.CDPRODUTO, IAF.DTPROGENTAF, IAF.IDBAIXITMAF,IT.NRAUTOFORN AS NRAUTOFORNE,
                   SUM(IAF.QTPROGENTAF) QTPROGENTAF, SUM(IAF.QTEFETENTRAF) QTEFETENTRAF
              FROM ITEMNOTAFISC I, NFRELACAF N, ITEMPRODUTO IT, ITEMAUTF IAF
             WHERE I.CDFILIAL = :P_CDFILIAL
               AND I.NRLANCTONF = :P_NRLANCTONF
               AND I.CDFILIALIT    = N.CDFILIAL
               AND I.NRLANCTONFIT = N.NRLANCTONF
               AND N.CDFILIAL     = IT.CDFILIAL
               AND N.NRLANCTONF   = IT.NRLANCTONF
               AND IT.VRUNITITEM  = :P_VRUNITITEM
               AND IT.NRAUTOFORN IS NOT NULL
               AND IT.CDPRODUTO = N.CDPRODUTO
               AND N.CDFILDEST = IAF.CDFILDEST
               AND N.CDFILLANC = IAF.CDFILLANC
               AND N.NRAUTOFORN = IAF.NRAUTOFORN
               AND N.CDMARCA = IAF.CDMARCA
               AND N.CDPRODUTO = IAF.CDPRODUTO
               AND N.CDPRODUTO = :P_CDPRODUTO
            GROUP BY N.CDFILDEST, N.CDFILLANC, N.NRAUTOFORN, N.CDMARCA,
                     N.CDPRODUTO, IAF.DTPROGENTAF, IAF.IDBAIXITMAF, IT.NRAUTOFORN
            ORDER BY N.CDPRODUTO, N.NRAUTOFORN DESC, QTPROGENTAF DESC,  IAF.DTPROGENTAF DESC
SQL;

            $params = array(
                'P_CDFILIAL' => $filial,
                'P_NRLANCTONF' => $lancamento,
                'P_VRUNITITEM' => $vrUnitario,
                'P_CDPRODUTO' => $produto
            );

            $query = $this->entityManager->getConnection()->prepare($sql);
            $query->execute($params);
            return $query->fetchAll();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function quantidadeProgramadaAFXML($filial, $lancamento, $vrUnitario, $produto) {
        try {
             $sql = <<<SQL
             SELECT N.CDFILDEST, N.CDFILLANC, N.NRAUTOFORN, N.CDMARCA,
                  N.CDPRODUTO, IAF.DTPROGENTAF, IAF.IDBAIXITMAF,IT.NRAUTOFORN AS NRAUTOFORNE,
                  SUM(IAF.QTPROGENTAF) QTPROGENTAF, SUM(IAF.QTEFETENTRAF) QTEFETENTRAF,
                  P.VRFATOCONV
             FROM ITEMNOTAFISC I, NFRELACAF N, ITEMPRODUTO IT, ITEMAUTF IAF, PRODUTO P, PRODUTO P1
            WHERE I.CDFILIAL     = :P_CDFILIAL
              AND I.NRLANCTONF   = :P_NRLANCTONF
              AND I.CDFILIALIT   = N.CDFILIAL
              AND I.NRLANCTONFIT = N.NRLANCTONF
              AND N.CDFILIAL     = IT.CDFILIAL
              AND N.NRLANCTONF   = IT.NRLANCTONF
              AND IAF.VRPREUNIAF = :P_VRUNITITEM
              AND IT.NRAUTOFORN IS NOT NULL
              AND IT.CDPRODUTO = P.CDPRODUTO
              AND P.CDPRODESTO =  P1.CDPRODESTO
              AND N.CDFILDEST  = IAF.CDFILDEST
              AND N.CDFILLANC  = IAF.CDFILLANC
              AND N.NRAUTOFORN = IAF.NRAUTOFORN
              AND N.CDMARCA    = IAF.CDMARCA
              AND N.CDPRODUTO  = IAF.CDPRODUTO
              AND N.CDPRODUTO  = P1.CDPRODUTO
              AND N.CDPRODUTO  = :P_CDPRODUTO
           GROUP BY N.CDFILDEST, N.CDFILLANC, N.NRAUTOFORN, N.CDMARCA,
                    N.CDPRODUTO, IAF.DTPROGENTAF, IAF.IDBAIXITMAF, IT.NRAUTOFORN,
                    P.VRFATOCONV
           ORDER BY N.CDPRODUTO, N.NRAUTOFORN DESC, QTPROGENTAF DESC,  IAF.DTPROGENTAF DESC
SQL;

            $params = array(
                'P_CDFILIAL' => $filial,
                'P_NRLANCTONF' => $lancamento,
                'P_VRUNITITEM' => $vrUnitario,
                'P_CDPRODUTO' => $produto
            );

            $query = $this->entityManager->getConnection()->prepare($sql);
            $query->execute($params);
            return $query->fetchAll();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function quantidadeEntradaAF($filial, $lancamento, $vrUnitario, $produto) {
        try {
             $sql = <<<SQL
             SELECT N.CDFILIAL, N.NRLANCTONF, SUM(N.QTITEMPROD) AS QTITEMPROD
              FROM (SELECT DISTINCT IT.CDFILIAL, IT.NRLANCTONF, IT.QTITEMPROD
                      FROM ITEMNOTAFISC I, NFRELACAF AF, NFRELACAF AF1, ITEMPRODUTO IT
                     WHERE I.CDFILIAL = :P_CDFILIAL
                       AND I.NRLANCTONF = :P_NRLANCTONF
                       AND I.CDFILIALIT = AF.CDFILIAL
                       AND I.NRLANCTONFIT = AF.NRLANCTONF
                       AND AF.CDPRODUTO = :P_CDPRODUTO
                       AND AF.CDFILDEST = AF1.CDFILDEST
                       AND AF.CDFILLANC = AF1.CDFILLANC
                       AND AF.NRAUTOFORN = AF1.NRAUTOFORN
                       AND AF.CDMARCA = AF1.CDMARCA
                       AND AF.CDPRODUTO = AF1.CDPRODUTO
                       AND AF.DTPROGENTAF = AF1.DTPROGENTAF
                       AND AF1.CDFILIAL = IT.CDFILIAL
                       AND AF1.NRLANCTONF = IT.NRLANCTONF
                       AND AF1.CDPRODUTO = IT.CDPRODUTO
                       AND IT.VRUNITITEM = :P_VRUNITITEM) N
            GROUP BY N.CDFILIAL, N.NRLANCTONF
SQL;

            $params = array(
                'P_CDFILIAL' => $filial,
                'P_NRLANCTONF' => $lancamento,
                'P_VRUNITITEM' => $vrUnitario,
                'P_CDPRODUTO' => $produto
            );

            $query = $this->entityManager->getConnection()->prepare($sql);
            $query->execute($params);
            return $query->fetchAll();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function quantidadeDevolvidaAF($filial, $lancamento, $vrUnitario, $produto) {
        try {
             $sql = <<<SQL
             SELECT N.CDFILIAL, N.NRLANCTONF, SUM(N.QTITEMPROD) QTITEMPROD
            FROM (SELECT DISTINCT IT.CDFILIAL, IT.NRLANCTONF, IT.QTITEMPROD
                    FROM ITEMNOTAFISC I, NFRELACAF AF, NFRELACAF AF1,ITEMNOTAFISC I1, ITEMPRODUTO IT
                   WHERE I.CDFILIAL = :P_CDFILIAL
                     AND I.NRLANCTONF = :P_NRLANCTONF
                     AND I.CDFILIALIT = AF.CDFILIAL
                     AND I.NRLANCTONFIT = AF.NRLANCTONF
                     AND AF.CDFILDEST = AF1.CDFILDEST
                     AND AF.CDFILLANC = AF1.CDFILLANC
                     AND AF.NRAUTOFORN = AF1.NRAUTOFORN
                     AND AF.CDMARCA = AF1.CDMARCA
                     AND AF.CDPRODUTO = AF1.CDPRODUTO
                     AND AF.DTPROGENTAF = AF1.DTPROGENTAF
                     AND AF1.CDFILIAL = I1.CDFILIALIT
                     AND AF1.NRLANCTONF =  I1.NRLANCTONFIT
                     AND I1.CDFILIAL = IT.CDFILIAL
                     AND I1.NRLANCTONF = IT.NRLANCTONF
                     AND IT.CDPRODUTO = :P_CDPRODUTO
                     AND IT.VRUNITITEM = :P_VRUNITITEM
                     AND I1.CDFILIAL||I1.NRLANCTONF <> :P_CDFILIAL||:P_NRLANCTONF) N
            GROUP BY N.CDFILIAL, N.NRLANCTONF
SQL;

            $params = array(
                'P_CDFILIAL' => $filial,
                'P_NRLANCTONF' => $lancamento,
                'P_VRUNITITEM' => $vrUnitario,
                'P_CDPRODUTO' => $produto
            );

            $query = $this->entityManager->getConnection()->prepare($sql);
            $query->execute($params);
            return $query->fetchAll();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function existeAF($filial, $lancamento, $produto) {
        try {
             $sql = <<<SQL
             SELECT I.CDPRODUTO, I.NRAUTOFORN
              FROM ITEMPRODUTO I, ITEMNOTAFISC IT, NFRELACAF NF
             WHERE IT.CDFILIAL     = :P_CDFILIAL
               AND IT.NRLANCTONF   = :P_NRLANCTONF
               AND I.CDPRODUTO     = :P_CDPRODUTO
               AND IT.CDFILIALIT   = I.CDFILIAL
               AND IT.NRLANCTONFIT = I.NRLANCTONF
               AND I.CDPRODUTO     = NF.CDPRODUTO
               AND I.CDFILIAL      = NF.CDFILIAL
               AND I.NRLANCTONF    = NF.NRLANCTONF
SQL;

            $params = array(
                'P_CDFILIAL' => $filial,
                'P_NRLANCTONF' => $lancamento,
                'P_CDPRODUTO' => $produto
            );

            $query = $this->entityManager->getConnection()->prepare($sql);
            $query->execute($params);
            return $query->fetchAll();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function existeAFXML($filial, $lancamento, $produto) {
        try {
             $sql = <<<SQL
             SELECT I.CDPRODUTO, I.NRAUTOFORN, NF.CDPRODUTO AS CDPRODUTOAF,
                       NF.VRNFRELACAF, N.IDNFGERADAVIAXML
                  FROM ITEMPRODUTO I, ITEMNOTAFISC IT, NFRELACAF NF, NOTAFISCAL N,
                       PRODUTO P, PRODUTO P1
                 WHERE IT.CDFILIAL     = :P_CDFILIAL
                   AND IT.NRLANCTONF   = :P_NRLANCTONF
                   AND IT.CDFILIALIT   = I.CDFILIAL
                   AND IT.NRLANCTONFIT = I.NRLANCTONF
                   AND I.CDFILIAL      = NF.CDFILIAL
                   AND I.NRLANCTONF    = NF.NRLANCTONF
                   AND I.CDPRODUTO     = :P_CDPRODUTO
                   AND P.CDPRODUTO     = I.CDPRODUTO
                   AND P.CDPRODESTO    = P1.CDPRODESTO
                   AND P1.CDPRODUTO    = NF.CDPRODUTO
                   AND N.CDFILIAL      = I.CDFILIAL
                   AND N.NRLANCTONF    = I.NRLANCTONF
SQL;

            $params = array(
                'P_CDFILIAL' => $filial,
                'P_NRLANCTONF' => $lancamento,
                'P_CDPRODUTO' => $produto
            );

            $query = $this->entityManager->getConnection()->prepare($sql);
            $query->execute($params);
            return $query->fetchAll();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function buscaAForigem($filial, $lancamento) {
        try {
             $sql = <<<SQL
             SELECT N.CDFILDEST, N.CDFILLANC, N.NRAUTOFORN, COUNT(*) AS QTITEM
               FROM ITEMNOTAFISC IT, NFRELACAF N
              WHERE IT.CDFILIAL = :P_CDFILIAL
                AND IT.NRLANCTONF = :P_NRLANCTONF
                AND IT.CDFILIALIT = N.CDFILIAL
                AND IT.NRLANCTONFIT = N.NRLANCTONF
              GROUP BY N.CDFILDEST, N.CDFILLANC, N.NRAUTOFORN
SQL;

            $params = array(
                'P_CDFILIAL' => $filial,
                'P_NRLANCTONF' => $lancamento,
            );

            $query = $this->entityManager->getConnection()->prepare($sql);
            $query->execute($params);
            return $query->fetchAll();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function buscaItensAForigem($filial, $lancamento) {
        try {
             $sql = <<<SQL
             SELECT N.CDFILDEST, N.CDFILLANC, N.NRAUTOFORN,
                    N.CDMARCA, N.CDPRODUTO, N.DTPROGENTAF
               FROM ITEMNOTAFISC IT, NFRELACAF N
              WHERE IT.CDFILIAL = :P_CDFILIAL
                AND IT.NRLANCTONF = :P_NRLANCTONF
                AND IT.CDFILIALIT = N.CDFILIAL
                AND IT.NRLANCTONFIT = N.NRLANCTONF
SQL;

            $params = array(
                'P_CDFILIAL' => $filial,
                'P_NRLANCTONF' => $lancamento,
            );

            $query = $this->entityManager->getConnection()->prepare($sql);
            $query->execute($params);
            return $query->fetchAll();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function updateItemAlteracaoNotaAF($organizacao, $operador, $itemautf, $status, $qtEntregue, $baixaFutura, $iddevolucaonfit){
        try {
            $data = DateUtil::getDataAtual();

            $itemautf->setIdbaixitmaf($status);
            $itemautf->setQtefetentraf($qtEntregue);
            $itemautf->setIddevolucaonfit($iddevolucaonfit);

            if($baixaFutura)
                $itemautf->setIdbaixitmfutura($baixaFutura);

            $itemautf->setNrorgultatu($organizacao);
            $itemautf->setCdoperultatu($operador);
            $itemautf->setDtultatu($data);

            $this->entityManager->persist($itemautf);
            $this->entityManager->flush();

            return $itemautf;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function updateQuantidadeItemNotaAF($organizacao, $operador, $itemautf, $quantidade, $iddevolucaonfit){
        try {
            $data = DateUtil::getDataAtual();

            $itemautf->setQtefetentraf($itemautf->getQtefetentraf() + $quantidade);
            $itemautf->setIddevolucaonfit($iddevolucaonfit);

            $itemautf->setNrorgultatu($organizacao);
            $itemautf->setCdoperultatu($operador);
            $itemautf->setDtultatu($data);

            $this->entityManager->persist($itemautf);
            $this->entityManager->flush();

            return $itemautf;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function updateBaixaItemNotaAF($organizacao, $operador, $itemautf, $baixa, $baixaFutura){
        try {
            $data = DateUtil::getDataAtual();

            $itemautf->setIdbaixitmaf($baixa);

            if($baixaFutura)
                $itemautf->setIdbaixitmfutura($baixaFutura);

            $itemautf->setNrorgultatu($organizacao);
            $itemautf->setCdoperultatu($operador);
            $itemautf->setDtultatu($data);

            $this->entityManager->persist($itemautf);
            $this->entityManager->flush();

            return $itemautf;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function callProcedure_ATUALIZA_QTRELAC_ITNFORIGEM($filial, $lancamento, $sequencialItem, $diferenca = 0, $status = 'A', $deletaNF = null) {
        try {
            $procedure = new StoredProcedure($this->entityManager->getConnection(), 'ATUALIZA_QTRELAC_ITNFORIGEM');
            $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $filial, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRLANCTONF', Param::PARAM_INPUT, $lancamento, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRSEQITPROD', Param::PARAM_INPUT, $sequencialItem, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_VRDIFERENCA', Param::PARAM_INPUT, $diferenca, Param::PARAM_TYPE_INT));
            $procedure->addParam(new Param('P_IDSTATUS', Param::PARAM_INPUT, $status, Param::PARAM_TYPE_STR));
            if($deletaNF)
                $procedure->addParam(new Param('P_DELETANF', Param::PARAM_INPUT, $deletaNF, Param::PARAM_TYPE_STR));
            $erro = $procedure->execute();
            return $erro;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function callProcedure_BAIXAAF($filialDestino, $filialLancamento, $af) {
        try {
            $procedure = new StoredProcedure($this->entityManager->getConnection(), 'BAIXAAF');
            $procedure->addParam(new Param('P_CDFILDEST', Param::PARAM_INPUT, $filialDestino, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFILLANC', Param::PARAM_INPUT, $filialLancamento, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRAUTOFORN', Param::PARAM_INPUT, $af, Param::PARAM_TYPE_STR));
            $erro = $procedure->execute();
            return $erro;
        } catch (\Exception $e) {
            throw $e;
        }
    }
}
