<?php

namespace Teknisa\OPL\Factories;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class ProceduresNfFactory
{
    private $entityManager;

    const PROCEDURE_CANCELA_DADOS_FINANCEIRO    = 'CANCELA_DADOS_FINANCEIRO';
    const PROCEDURE_CONSULTA_GRUPO_FISCAL       = 'CONSULTA_GRUPOFISCAL';
    const PROCEDURE_CTB_DFE_CONTAB              = 'CTB_DFE_CONTAB';
    const PROCEDURE_CTB_DFS_CONTAB              = 'CTB_DFS_CONTAB';
    const PROCEDURE_CTB_DFE_AJUSTE              = 'CTB_DFE_AJUSTE';
    const PROCEDURE_CUSTO_LIQUIDO               = 'CUSTO_LIQUIDO';
    const PROCEDURE_DELETA_NF                   = 'DELETA_NF';
    const PROCEDURE_GERA_DADOS_FINANCEIRO       = 'GERA_DADOS_FINANCEIRO';
    const PROCEDURE_GRAVA_IMPOSTO               = 'GRAVA_IMPOSTO';
    const PROCEDURE_IMPOSTO_SAIDA               = 'IMPOSTO_SAIDA';
    const PROCEDURE_IMPOSTO_ENTRADA             = 'IMPOSTO_ENTRADA';
    const PROCEDURE_IMPOSTO_ENTRADA_PRODUTO     = 'IMPOSTO_ENTRADA_PRODUTO';
    const PROCEDURE_REPLICACAO_DOCUMENTOS       = 'REPLICACAO_DOCUMENTOS';

    public function __construct(EntityManager $entityManager)
    {
        $this->entityManager = $entityManager;
    }

    public function procedureGeraDadosFinanceiro($filial, $lancamento, $operacao, $filiemit){
        try {
            $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_GERA_DADOS_FINANCEIRO);
            $procedure->addParam(new Param('P_CDFILIAL',    Param::PARAM_INPUT, $filial,        Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRLANCTONF',  Param::PARAM_INPUT, $lancamento,    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFILIPROC',  Param::PARAM_INPUT, $filiemit,        Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDOPERACAO',  Param::PARAM_INPUT, $operacao,      Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_RESULTADO',   Param::PARAM_OUTPUT, '',            Param::PARAM_TYPE_STR, 10));
            return $procedure->execute();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function procedureContabilizaNotaFiscalEntrada($filial, $lancamento){
        try {
            $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_CTB_DFE_CONTAB);
            $procedure->addParam(new Param('P_CDFILIAL',    Param::PARAM_INPUT, $filial,        Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRLANCTONF',  Param::PARAM_INPUT, $lancamento,    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRLANCCTB',   Param::PARAM_OUTPUT, '',            Param::PARAM_TYPE_STR, 12));
            return $procedure->execute();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function procedureContabilizaNotaFiscalSaida($filial, $lancamento){
        try {
            $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_CTB_DFS_CONTAB);
            $procedure->addParam(new Param('P_CDFILIAL',    Param::PARAM_INPUT, $filial,        Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRLANCTONF',  Param::PARAM_INPUT, $lancamento,    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRLANCCTB',   Param::PARAM_OUTPUT, '',            Param::PARAM_TYPE_STR, 12));
            return $procedure->execute();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function procedureAjuste($cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag){
        try {
            $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_CTB_DFE_AJUSTE);
            $procedure->addParam(new Param('P_CDFILIAL',        Param::PARAM_INPUT, $cdfilial,      Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRLANCTONF',      Param::PARAM_INPUT, $nrlanctonf,    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRSEQVENPAG',     Param::PARAM_INPUT, $nrseqvenpag,   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRPARCELAPAG',    Param::PARAM_INPUT, $nrparcelapag,  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRLANCCTB',       Param::PARAM_OUTPUT, '',            Param::PARAM_TYPE_STR, 12));
            return $procedure->execute();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function procedureCancelaDadosFinanceiro($cdfilial, $nrlanctonf, $idcanfin = 'S'){
        try {
            $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_CANCELA_DADOS_FINANCEIRO);
            $procedure->addParam(new Param('P_CDFILIAL',    Param::PARAM_INPUT, $cdfilial,  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRLANCTONF', Param::PARAM_INPUT, $nrlanctonf, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDCANFIN',    Param::PARAM_INPUT, $idcanfin,  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_RETORNO',     Param::PARAM_OUTPUT, '',        Param::PARAM_TYPE_INT, 1));
            return $procedure->execute();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function deletaNf($filial, $lancamento, $corpoItem = null){
        try {
            $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_DELETA_NF);
            $procedure->addParam(new Param('P_FILIAL',      Param::PARAM_INPUT, $filial,        Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_LANCTONF',    Param::PARAM_INPUT, $lancamento,    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDCORPITEM',  Param::PARAM_INPUT, $corpoItem,     Param::PARAM_TYPE_STR));
            $procedure->execute();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function replicacaoDocumentos($filial, $lancamento, $filialLogada, $proxCodNota, $proxCodEstoque, $iddocfatur){
        try {
            $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_REPLICACAO_DOCUMENTOS);
            $procedure->addParam(new Param('P_CDFILIAL',    Param::PARAM_INPUT, $filial,            Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRLANCTONF',  Param::PARAM_INPUT, $lancamento,        Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('CDFILIPROC',    Param::PARAM_INPUT, $filialLogada,      Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_PRXNOTAFISC', Param::PARAM_INPUT, $proxCodNota,       Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_PRXLANCETSQ', Param::PARAM_INPUT, $proxCodEstoque,    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDDOCFATUR',  Param::PARAM_INPUT, $iddocfatur,        Param::PARAM_TYPE_STR));
            $procedure->execute();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function procedureCustoLiquido($filial, $lancamento, $totalProd, $direfenca = null, $gravvrbrut = null) {
        try {
            $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_CUSTO_LIQUIDO);
            $procedure->addParam(new Param('P_FILIAL',      Param::PARAM_INPUT, $filial, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_LANCTO',      Param::PARAM_INPUT, $lancamento, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_TOTALPROD',   Param::PARAM_INPUT, $totalProd, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DIFERENCA',   Param::PARAM_INPUT, $direfenca, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_GRAVVRBRUT',  Param::PARAM_INPUT, $gravvrbrut, Param::PARAM_TYPE_STR));
            $procedure->execute();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function procedureImpostoEntradaProduto( $empresa, $inscesta, $tipoOperacao, $fornecedor, $filial,
            $seqLancamento, $dtEntrada, $entrada_saida, $operador, $grupofiscal,
            $idinterest, $cdarvprod, $cdgrpfiscsv, $idorigdestnf, $filialRece, $cdfiliemit,
            $cliente, $valorProduto, $idorigemprod, $sgestado, $cdmunicipio ) {

        $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_IMPOSTO_ENTRADA_PRODUTO);
        $procedure->addParam(new Param('P_EMPRESA',         Param::PARAM_INPUT, $empresa,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_INSCESTA',        Param::PARAM_INPUT, $inscesta,      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_TIPOOPER',        Param::PARAM_INPUT, $tipoOperacao,  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFORNECED',      Param::PARAM_INPUT, $fornecedor,    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_FILIAL',          Param::PARAM_INPUT, $filial,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_LANCTONF',        Param::PARAM_INPUT, $seqLancamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_INIVALID',        Param::PARAM_INPUT, $dtEntrada,     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_ENTRSAID',        Param::PARAM_INPUT, $entrada_saida, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR',      Param::PARAM_INPUT, $operador,      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDGRUPOFISC',     Param::PARAM_INPUT, $grupofiscal,   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDINTEREST',      Param::PARAM_INPUT, $idinterest,    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDARVPROD',       Param::PARAM_INPUT, $cdarvprod,     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDGRPFISCSV',     Param::PARAM_INPUT, $cdgrpfiscsv,   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDORIGDESTNF',    Param::PARAM_INPUT, $idorigdestnf,  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIRECE',      Param::PARAM_INPUT, $filialRece,    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIEMIT',      Param::PARAM_INPUT, $cdfiliemit,    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCLIENTE',       Param::PARAM_INPUT, $cliente,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRPRODUTO',       Param::PARAM_INPUT, $valorProduto,  Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param('P_IDORIGEMPROD',    Param::PARAM_INPUT, $idorigemprod,  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_SGESTADO',        Param::PARAM_INPUT, $sgestado,      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDMUNICIPIO',     Param::PARAM_INPUT, $cdmunicipio,   Param::PARAM_TYPE_STR));
        $procedure->execute();
        
    }
    
     public function procedureImpostoEntrada( $empresa, $inscesta, $tipoOperacao, $forncli, $filial,
            $seqLancamento, $dtEntrada, $operador, $nrlancestoq = '', $entrsaid = 'E', $audit = 'S') {
                
        $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_IMPOSTO_ENTRADA);
        $procedure->addParam(new Param('P_EMPRESA',         Param::PARAM_INPUT, $empresa,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_INSCESTA',        Param::PARAM_INPUT, $inscesta,      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_TIPOOPER',        Param::PARAM_INPUT, $tipoOperacao,  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_FORNECED',        Param::PARAM_INPUT, $forncli,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_FILIAL',          Param::PARAM_INPUT, $filial,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_LANCTONF',        Param::PARAM_INPUT, $seqLancamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_INIVALID',        Param::PARAM_INPUT, $dtEntrada,     Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param("P_VRNOTAFISC",      Param::PARAM_OUTPUT, '',            Param::PARAM_TYPE_STR, 20));
        
        $procedure->addParam(new Param("P_LANCESTQ",        Param::PARAM_INPUT, $nrlancestoq,  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_ENTRSAID",        Param::PARAM_INPUT, $entrsaid,     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDOPERADOR",      Param::PARAM_INPUT, $operador,     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_AUDIT",           Param::PARAM_INPUT, $audit,        Param::PARAM_TYPE_STR));
        $output = $procedure->execute();

        return (float) $output['P_VRNOTAFISC'];
    }
    
    public function procedureImpostoSaida( $empresa, $inscesta, $tipoOperacao, $forncli, $filial,
            $seqLancamento, $dtEntrada, $operador, $nrlancestoq = '') {
                
        $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_IMPOSTO_SAIDA);
        $procedure->addParam(new Param('P_EMPRESA',         Param::PARAM_INPUT, $empresa,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_INSCESTA',        Param::PARAM_INPUT, $inscesta,      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_TIPOOPER',        Param::PARAM_INPUT, $tipoOperacao,  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CLIENTE',         Param::PARAM_INPUT, $forncli,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_FILIAL',          Param::PARAM_INPUT, $filial,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_LANCTONF',        Param::PARAM_INPUT, $seqLancamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_INIVALID',        Param::PARAM_INPUT, $dtEntrada,     Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param("P_VRNOTAFISC",      Param::PARAM_OUTPUT, '',            Param::PARAM_TYPE_STR, 20));
        
        $procedure->addParam(new Param("P_LANCESTQ",        Param::PARAM_INPUT, $nrlancestoq,  Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param("P_CDOPERADOR",      Param::PARAM_INPUT, $operador,     Param::PARAM_TYPE_STR));
        
        $output = $procedure->execute();

        return (float) $output['P_VRNOTAFISC'];
    }
    
    public function procedureConsultaGrupofiscal($idorigdestnf, $fornecedor, $filial, $entrada_saida, $tipoOperacao) {
        $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_CONSULTA_GRUPO_FISCAL);
        $procedure->addParam(new Param("P_IDORIGDESTNF",    Param::PARAM_INPUT, $idorigdestnf,  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDORIGDESTNF",    Param::PARAM_INPUT, $fornecedor,    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDFILIFATUR",     Param::PARAM_INPUT, $filial,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_IDENTRSAIDOP",    Param::PARAM_INPUT, $entrada_saida, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDTIPOOPER",      Param::PARAM_INPUT, $tipoOperacao,  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDGRUPOFISC",     Param::PARAM_OUTPUT, '',      Param::PARAM_TYPE_STR, 10));
        $output = $procedure->execute();

        return $output['P_CDGRUPOFISC'];
    }
    
    public function procedureGravaImposto($filial, $lancamento) {
        $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_GRAVA_IMPOSTO);
        $procedure->addParam(new Param("P_FILIAL",          Param::PARAM_INPUT, $filial,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_LANCTONF",        Param::PARAM_INPUT, $lancamento,    Param::PARAM_TYPE_STR));
        return $procedure->execute();


    }
}