<?php

namespace Teknisa\OPL\Services;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\OPL\Factories\Dfs48000SaidaDoOperadorLogisticoFactory;

use Teknisa\OPL\Services\LancamentoNotafiscalService;
use Teknisa\PebLibs\Services\ValidacoesEstoqueService;

use Teknisa\Libs\Util\Masks\PictureProduto; 
use Teknisa\Libs\Util\Masks\Masks;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\Environment;

use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
    
class Dfs48000SaidaDoOperadorLogisticoService {
    
    const PROCEDURE_GERA_PRODUTO            = 'GERA_NOTAFISCAL_PRODUTO';
    const PROCEDURE_ATUALIZA_ESTOQUE        = 'ATUALIZA_ESTOQUE';
    const NOVO_CODIGO_MOVOCORR              = 'MOVOCORR';
    const NOVO_CODIGO_LANC_ESTOQUE          = 'LANCTOEST';
    const NOVO_CODIGO_NOTA_LANC             = 'LANCNF/FILIAL';
    const NOVO_CODIGO_NRPEXOL               = 'PEDIDOEXOL';
    const PROCEDURE_GERA_NOTA_CORPO_ENTRADA = 'GERA_NOTAFISCAL_CORPO_ENTRADA';
    const PROCEDURE_GERA_NOTA_CORPO         = 'GERA_NOTAFISCAL_CORPO';
    const PROCEDURE_IMPOSTO_ENTRADA_PRODUTO = 'IMPOSTO_ENTRADA_PRODUTO';
    const PROCEDURE_GERA_FECHAMENTO_ENTRADA = 'GERA_FECHAMENTO_NFENTRADA';
    
    const MSG_ERROS = 
        array("DAT_MAI"          => "Data informada não deve ser maior do que a data de processamento",
              "DAT_MEN"          => "Data informada não deve ser menor do que a data de processamento",
              "DAT_M"            => "Data informada fora do período permitido.",
              "SERIE_PAD"        => "Não existe série padrão para filial.",
              "LANC_FORTADT"     => "Data de saída fora do periodo de integração.",
              "LANC_N_GER"       => "Não existem itens cuja quantidade devolvida é maior do que 0.",
              "TOTAL_NF_ZERADO"  => "Existem itens cujo total da NF de saída está zerado.",
              "DTEMISSAO_VAZIA"  => "Nota Fiscal ainda não foi aprovada(sem data de emissão).",
              "NRNOTAFISC_VAZIO" => "Nota Fiscal não pode ter data de saída menor do que a data de emissão de saída.", 
              "QTDE_ZERADA"      => "Quantidade do item devolvida não foi informada ou está zerada.",
              "PRECO_ZERADO"     => "Existem itens cujo preço ficaria zerado.",
              "PRODUTO_FILIAL"   => "Produto não existe para a filial.",
              "PRODUTO_INATIVO"  => "Produto inativo no sistema.",
              "PRODUTO_INATFIL"  => "Produto inativo para filial.",
              "EST_INSUFICIENTE" => "Estoque do item insuficiente para devolução.",
              "ERRO_TOT"         => "Valor total da NF de saÍda diferente do valor total da NF de saída.",
              "SEMDOCUFISC"      => "Modelo de documento fiscal não cadastrado.",
              "ITEMSEMCFOP"      => "CFOP não parametrizado para o item.");
              
    protected $dfs48000SaidaDoOperadorLogisticoService;
    protected $NewCode;
    protected $lancamentoNotaFiscalService;
    
    private $arrayNotasErro    = array();
    private $arrayProdutosErro = array();
    private $arrayNotasSucesso = array();
    private $notasSucesso      = 0;
    
    public function __construct(Dfs48000SaidaDoOperadorLogisticoFactory $dfs48000SaidaDoOperadorLogisticoFactory, EntityManager $entityManager, NewCode $NewCode, LancamentoNotafiscalService $lancamentoNotaFiscalService, Environment $environment, ValidacoesEstoqueService $validacoesEstoqueService) {
        $this->dfs48000SaidaDoOperadorLogisticoFactory = $dfs48000SaidaDoOperadorLogisticoFactory;
        $this->connection   = $entityManager->getConnection();
        $this->NewCode   = $NewCode;
        $this->lancamentoNotaFiscalService = $lancamentoNotaFiscalService;
        $this->validacoesEstoqueService    = $validacoesEstoqueService;
        $this->environment  = $environment;
        $this->masks = new Masks();
    }
    
    
    public function geraCorpoNotaEntrada($cdfilial, $cdfilirece, $cdempresa, $cdtipooper, $cdserieentr, $cdcliente, $sistema, $atualizaestoque, $nrlanctonf, $pesstrans, $insctrans, $placaveiculo, $ufveiculo, $pgtofrete, $quantidadedevolu, $espefvolu, $marcvolu, $numevolu, $nrlancestoq, $dtentrsaid, $vrbrutovolu, $vrliquidovolu, $vrnotafisc, $nrlancorig, $dtlancmennf, $dtemissao, $nrnotafisc, $nracessonfe, $cdmododocfisc) {
        $dtemissao  = str_replace("-", "/", $dtemissao);
        $dtentrsaid = str_replace("-", "/", $dtentrsaid);
        $prxLanctoNf    = NULL;
        $prxLanctoEstoq = NULL;
        
        $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_GERA_NOTA_CORPO_ENTRADA);
        $procedure->addParam(new Param('P_CDFILIAL',        Param::PARAM_INPUT, $cdfilial,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIRECE',      Param::PARAM_INPUT, $cdfilirece,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDEMPRESA',       Param::PARAM_INPUT, $cdempresa,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDTIPOOPER',      Param::PARAM_INPUT, $cdtipooper,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSERIEENTR',     Param::PARAM_INPUT, $cdserieentr,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCLIENTE',       Param::PARAM_INPUT, $cdcliente,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTENTRSAID',      Param::PARAM_INPUT, $dtentrsaid,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDSISTEMA',       Param::PARAM_INPUT, $sistema,           Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRNOTAFISC',      Param::PARAM_INPUT, $vrnotafisc,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_PRXLANCTONF',     Param::PARAM_OUTPUT,$prxLanctoNf,       Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_PRXLANCESTQ',     Param::PARAM_OUTPUT,$prxLanctoEstoq,    Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_ATUAESTOQUE',     Param::PARAM_INPUT, $atualizaestoque,   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRAPURACFAT',     Param::PARAM_INPUT, NULL,               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF',      Param::PARAM_INPUT, $nrlanctonf,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDTPIJURTRAN',    Param::PARAM_INPUT, $pesstrans,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRINSJURTRAN',    Param::PARAM_INPUT, $insctrans,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRPLACAVEIC',     Param::PARAM_INPUT, $placaveiculo,      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_SGUFVEIC',        Param::PARAM_INPUT, $ufveiculo,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDPGTOFRETE',     Param::PARAM_INPUT, $pgtofrete,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSQTDEVOLU',      Param::PARAM_INPUT, $quantidadedevolu,  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSESPECVOLU',     Param::PARAM_INPUT, $espefvolu,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSMARCAVOLU',     Param::PARAM_INPUT, $marcvolu,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSNUMEVOLU',      Param::PARAM_INPUT, $numevolu,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRPBRUTOVOLU',    Param::PARAM_INPUT, $vrbrutovolu,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRPLIQUVOLU',     Param::PARAM_INPUT, $vrliquidovolu,     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCESTQ',      Param::PARAM_INPUT, $nrlancestoq,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCORIG',      Param::PARAM_INPUT, $nrlancorig,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTLANCAMENNF',    Param::PARAM_INPUT, NULL,               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRNOTAFISC',      Param::PARAM_INPUT, $nrnotafisc,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTEMISSAO',       Param::PARAM_INPUT, $dtemissao,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRACESSONFE',     Param::PARAM_INPUT, $nracessonfe,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDMODDOCFISC',    Param::PARAM_INPUT, $cdmododocfisc,     Param::PARAM_TYPE_STR));
        
        return $procedure->execute();
    }
    
    public function geraCorpoNota($cdfilial, $cdfiliemit, $cdempresa, $cdtipooper, $cdserie, $cdcliente, $sistema, $atualizaestoque, $nrlanctonf, $pesstrans, $insctrans, $placaveiculo, $ufveiculo, $pgtofrete, $quantidadedevolu, $espefvolu, $marcvolu, $numevolu, $nrlancestoq, $dtentrsaid, $vrbrutovolu, $vrliquidovolu, $nrlancorig = null, $dtlancmennf = null) {
        $dtentrsaid = str_replace("-", "/", $dtentrsaid);
        $cdtipooper = $cdtipooper['CDTIPOOPER'];
        
        $procedure = new StoredProcedure($this->connection, 'GERA_NOTAFISCAL_CORPO');
        $procedure->addParam(new Param('P_CDFILIAL',        Param::PARAM_INPUT, $cdfilial,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIEMIT',      Param::PARAM_INPUT, $cdfiliemit,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDEMPRESA',       Param::PARAM_INPUT, $cdempresa,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDTIPOOPER',      Param::PARAM_INPUT, $cdtipooper,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSERIE',         Param::PARAM_INPUT, $cdserie,           Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCLIENTE',       Param::PARAM_INPUT, $cdcliente,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTENTRSAID',      Param::PARAM_INPUT, $dtentrsaid,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDSISTEMA',       Param::PARAM_INPUT, $sistema,           Param::PARAM_TYPE_STR));

        $procedure->addParam(new Param('P_PRXLANCTONF',     Param::PARAM_OUTPUT, NULL,              Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_PRXLANCESTQ',     Param::PARAM_OUTPUT, NULL,              Param::PARAM_TYPE_STR, 10));
        
        $procedure->addParam(new Param('P_ATUAESTOQUE',     Param::PARAM_INPUT, $atualizaestoque,   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRAPURACFAT',     Param::PARAM_INPUT, NULL,               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF',      Param::PARAM_INPUT, $nrlanctonf,        Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param('P_IDTPIJURTRAN',    Param::PARAM_INPUT, $pesstrans,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRINSJURTRAN',    Param::PARAM_INPUT, $insctrans,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRPLACAVEIC',     Param::PARAM_INPUT, $placaveiculo,      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_SGUFVEIC',        Param::PARAM_INPUT, $ufveiculo,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDPGTOFRETE',     Param::PARAM_INPUT, $pgtofrete,         Param::PARAM_TYPE_STR));
        
        
        $procedure->addParam(new Param('P_DSQTDEVOLU',      Param::PARAM_INPUT, $quantidadedevolu,  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSESPECVOLU',     Param::PARAM_INPUT, $espefvolu,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSMARCAVOLU',     Param::PARAM_INPUT, $marcvolu,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSNUMEVOLU',      Param::PARAM_INPUT, $numevolu,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRPBRUTOVOLU',    Param::PARAM_INPUT, $vrbrutovolu,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRPLIQUVOLU',     Param::PARAM_INPUT, $vrliquidovolu,     Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param('P_NRLANCESTQ',      Param::PARAM_INPUT, $nrlancestoq,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCORIG',      Param::PARAM_INPUT, $nrlancorig,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTLANCAMENNF',    Param::PARAM_INPUT, $dtlancmennf,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDGEROUTDOCU',    Param::PARAM_INPUT, NULL,               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTCOMPFATUR',     Param::PARAM_INPUT, NULL,               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDPTOVNDFATURNF', Param::PARAM_INPUT, NULL,               Param::PARAM_TYPE_STR));
        
        return $procedure->execute();
    }
    
    public function procedureGeraNotaFiscalProduto($filial, $numLancamento, $produto, $codEmissao, $descEmissao, $quantidade, $vrUnitario, $vrDesconto, $almoxarifado = null, $localEstoque = null, $nrloteestq = null, $nrsublote = null, $cdservfatu = null, $idOrigemProd = 'N', $idAtuEstProd = 'S',  $vrICMSItOrig = 0, $cdMarca = null) {
        
        $procedure = new StoredProcedure($this->connection, self::PROCEDURE_GERA_PRODUTO);
        
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $filial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF', Param::PARAM_INPUT, $numLancamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDPRODUTO', Param::PARAM_INPUT, $produto, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDPRODEMIS', Param::PARAM_INPUT, $codEmissao, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSPRODEMIS', Param::PARAM_INPUT, $descEmissao, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_QTITEMPROD', Param::PARAM_INPUT, $quantidade, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRUNITITEM', Param::PARAM_INPUT, $vrUnitario, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRDESCITEM', Param::PARAM_INPUT, $vrDesconto, Param::PARAM_TYPE_STR));

        $procedure->addParam(new Param('P_CDFILIALITG', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRINTEGRACAO', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRITEMINTEGR', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCSTINFUSU', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCFOPINFUSU', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSUFXINFUSU', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));

        $procedure->addParam(new Param('P_CDALMOXARIFE', Param::PARAM_INPUT, $almoxarifado, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDLOCALESTOQ', Param::PARAM_INPUT, $localEstoque, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLOTEESTQ', Param::PARAM_INPUT, $nrloteestq, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRSUBLOTE', Param::PARAM_INPUT, $nrsublote, Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param('P_IDCONFORITEM', Param::PARAM_INPUT, 'S', Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDTPNFCREDEB', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_QTVOLUITEM', Param::PARAM_INPUT, 0, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSERVFILI', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIAPUR', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDMARCA', Param::PARAM_INPUT, $cdMarca, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSERVFATU', Param::PARAM_INPUT, $cdservfatu, Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param('P_IDORIGEMPROD', Param::PARAM_INPUT, $idOrigemProd, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDATUESTPROD', Param::PARAM_INPUT, $idAtuEstProd, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRICMSSTITORIG', Param::PARAM_INPUT, $vrICMSItOrig, Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param('P_DSJUSTIFICATIVA', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSINFADICPROD', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        
        $procedure->execute();
    }
    
    private function callProcedureImpostoEntrada($empresa, $inscesta, $tipoOperacao, $fornecedor, $filial, $seqLancamento, $dtEntrada, $entrada_saida, $operador, $grupofiscal, $idinterest, $cdarvprod, $cdgrpfiscsv, $idorigdestnf, $filialRece, $cdfiliemit, $cliente, $valorProduto) {
            
        $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_IMPOSTO_ENTRADA_PRODUTO);
        $procedure->addParam(new Param('P_EMPRESA',         Param::PARAM_INPUT, $empresa,           Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_INSCESTA',        Param::PARAM_INPUT, $inscesta,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_TIPOOPER',        Param::PARAM_INPUT, $tipoOperacao,      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFORNECED',      Param::PARAM_INPUT, $fornecedor,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_FILIAL',          Param::PARAM_INPUT, $filial,            Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_LANCTONF',        Param::PARAM_INPUT, $seqLancamento,     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_INIVALID',        Param::PARAM_INPUT, $dtEntrada,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_ENTRSAID',        Param::PARAM_INPUT, $entrada_saida,     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR',      Param::PARAM_INPUT, $operador,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDGRUPOFISC',     Param::PARAM_INPUT, $grupofiscal,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDINTEREST',      Param::PARAM_INPUT, $idinterest,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDARVPROD',       Param::PARAM_INPUT, $cdarvprod,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDGRPFISCSV',     Param::PARAM_INPUT, $cdgrpfiscsv,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDORIGDESTNF',    Param::PARAM_INPUT, $idorigdestnf,      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIRECE',      Param::PARAM_INPUT, $filialRece,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIEMIT',      Param::PARAM_INPUT, $cdfiliemit,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCLIENTE',       Param::PARAM_INPUT, $cliente,           Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRPRODUTO',       Param::PARAM_INPUT, $valorProduto,      Param::PARAM_TYPE_STR));
        
        $procedure->execute();
    }
    
    private function procedureGeraFechamentoEntrada($filial, $numLancamento, $dtProcessamento, $numLancEstoque, $operador = '') {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        $organizacao = $this->environment->getNrOrgTrab();
        $notaFiscal = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg($filial, $numLancamento, $organizacao);
        $this->entityManager->refresh($notaFiscal);
        $notaFiscal->setIdstatus('P');
        $notaFiscal->persist();
        $this->entityManager->flush();
        $connection->commit();
            
        $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_GERA_FECHAMENTO_ENTRADA);
        $procedure->addParam(new Param('P_CDFILIAL',        Param::PARAM_INPUT, $filial,            Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF',      Param::PARAM_INPUT, $numLancamento,     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DATAPROC',        Param::PARAM_INPUT, $dtProcessamento,   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCESTQ',      Param::PARAM_INPUT, $numLancEstoque,    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR',      Param::PARAM_INPUT, $operador,          Param::PARAM_TYPE_STR));
        
        $procedure->execute();
    }
    
    public function retornaSerieParaSaida($cdfilial) {
        try {
            $params = array(
                "P_CDFILIAL"  => $cdfilial
            );
            $query = "SELECT S.CDSERIE, IDCONMAXITEM, NRMAXITNF, S.CDEMPRESA
                        FROM SERIENF S, FILIAL F
                        WHERE F.CDFILIAL   = :P_CDFILIAL  AND
                              F.CDEMPRESA  = S.CDEMPRESA  AND
                              F.CDINSCESTA = S.CDINSCESTA AND
                              F.CDINSCMUNI = S.CDINSCMUNI AND
                              S.CDSERIE    = F.CDSERIE";
            $std = $this->connection->prepare($query);
            $std->execute($params);
                
            $resultado = $std->fetchAll();
                
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
 
    public function gerarSaida($notasFiscais, $dadosDevolucao, $dadosAdicionais = array()) {
        $cdFilEmit  = $notasFiscais[0]["CDFILDEST"];
        $cdFilDest  = $notasFiscais[0]["CDFILDEST"];
        
        $dadosFilialOpLog   = $this->dfs48000SaidaDoOperadorLogisticoFactory->getDadosFilial($dadosDevolucao['CDFILIOPELOG']);
        $dadosFilialDest    = $this->dfs48000SaidaDoOperadorLogisticoFactory->getDadosFilial($cdFilDest);
        $paramsGeracao      = $this->dfs48000SaidaDoOperadorLogisticoFactory->getParamsGeracao($dadosDevolucao['CDFILIOPELOG']);
        
        $cdOperador = $this->environment->getCdOperador();
        $nmOperador = $this->dfs48000SaidaDoOperadorLogisticoFactory->getNmOperador($cdOperador);
        
        $nmFilial   = $dadosFilialOpLog["NMFILIAL"];
        $cdEmpresa  = $dadosFilialOpLog["CDEMPRESA"];
        $cdInScesta = $dadosFilialOpLog["CDINSCESTA"];
        $cdSerie    = empty($dadosDevolucao["CDSERIE"]) ? $dadosDevolucao["CDSERIE"] : $dadosFilialDest["CDSERIE"];
        
        if($cdSerie == NULL) {
            $retornaSerie = self::retornaSerieParaSaida($dadosDevolucao['CDFILIOPELOG']);
            
            if(!empty($retornaSerie)) {
                $cdSerie = $retornaSerie[0]['CDSERIE'];
            }
        }
        
        $organizacao = $this->environment->getNrOrgTrab();
        
        $geraNota = true;
        
        foreach($notasFiscais as $notaFiscal) {
            try {
                $this->connection->beginTransaction();
                
                $nrLanctoNfNota = $this->NewCode->getCodeByTableName(self::NOVO_CODIGO_NOTA_LANC.$dadosDevolucao['CDFILIOPELOG'], $organizacao);    //LANCNF/FILIAL
                $nrLancEstoq    = $this->NewCode->getCodeByTableName(self::NOVO_CODIGO_LANC_ESTOQUE.$dadosDevolucao['CDFILIOPELOG'], $organizacao); //LANCTOEST
                
                $notaFiscal['NRLANCTONF'] = $nrLanctoNfNota;
                $notaFiscal['DTENTRSAID'] = $dadosDevolucao['DTENTREGA'];
                
                $proxLancamento = $this->geraCorpoNota($dadosDevolucao['CDFILIOPELOG'], $dadosDevolucao['CDFILIOPELOG'], $paramsGeracao["CDEMPRESA"], $dadosDevolucao["CDTIPOOPER"], $cdSerie, $cdFilEmit, 'DFS', 'S', $nrLanctoNfNota, '', '', '', '', $dadosDevolucao['FRETE']['CDFRETE'], '', '', '', '', $nrLancEstoq, $dadosDevolucao["DATA"], 0, 0, '', null);
                
                $this->insereDadosAdicionais($notaFiscal, $proxLancamento, $cdFilDest, $cdFilEmit, $dadosDevolucao, $organizacao, $paramsGeracao);
                
                $produtos = $this->validaItensPedido($notaFiscal, $dadosDevolucao);
                
                if(empty($produtos)) {
                    throw new \Exception(self::MSG_ERROS["LANC_N_GER"]);
                }
                
                $loopItem = 0;
                
                foreach ($produtos as $produto) {
                    $loopItem += 1;
                    $cdProdEmis = $this->maskProduto($produto["CDARVPRODEST"], $organizacao);
                        
                    try {
                        $adqVerifDuplNF = self::adqVerifDuplNF($dadosDevolucao['CDFILIOPELOG'], $cdFilDest, $this->environment->getCdFilial(), $produto);
                        
                        if(!empty($adqVerifDuplNF)) {
                            throw new \Exception('Nota Fiscal Duplicada.');
                        } else {
                            if($produto['IDUTILITEM'] == 1) {
                                 $adqNFRelacAFSai = self::adqNFRelacAFSai($dadosDevolucao['CDFILIOPELOG'], $nrLanctoNfNota, $cdFilDest, $this->environment->getCdFilial(), $produto);
                            
                                if(empty($adqNFRelacAFSai)) {
                                    self::acmInsNfRelacAfSai($dadosDevolucao['CDFILIOPELOG'], $this->environment->getCdFilial(), $nrLanctoNfNota, $cdFilDest, $produto);
                                }
                            } else {
                                if($dadosDevolucao['DATAENT'] == 0) {
                                    $adqNFRELACPEXOL = self::adqNFRELACPEXOL($dadosDevolucao['CDFILIOPELOG'], $this->environment->getCdFilial(), $nrLanctoNfNota, $cdFilDest, $produto);
                                
                                    if(empty($adqNFRELACPEXOL)) {
                                        self::acmInsNFRELACPEXOL($dadosDevolucao['CDFILIOPELOG'], $this->environment->getCdFilial(), $nrLanctoNfNota, $cdFilDest, $produto, null, $loopItem);
                                    }
                                } else {
                                    $adqExistItem = self::adqExistItem($cdFilDest, $this->environment->getCdFilial(), $produto);
                                    
                                    if(!empty($adqExistItem)) {
                                        $adqNFRELACPEXOL = self::adqNFRELACPEXOL($dadosDevolucao['CDFILIOPELOG'], $this->environment->getCdFilial(), $nrLanctoNfNota, $cdFilDest, $produto, $adqExistItem[0]['DTPROGENPEOL']);
                                    
                                        if(empty($adqNFRELACPEXOL)) {
                                            self::acmInsNFRELACPEXOL($dadosDevolucao['CDFILIOPELOG'], $this->environment->getCdFilial(), $nrLanctoNfNota, $cdFilDest, $produto, null, $loopItem);
                                        }
                                    } else {
                                        $adqNFRELACPEXOL = self::adqNFRELACPEXOL($dadosDevolucao['CDFILIOPELOG'], $this->environment->getCdFilial(), $nrLanctoNfNota, $cdFilDest, $produto);
                                        
                                        if(empty($adqNFRELACPEXOL)) {
                                            self::acmInsNFRELACPEXOL($dadosDevolucao['CDFILIOPELOG'], $this->environment->getCdFilial(), $nrLanctoNfNota, $cdFilDest, $produto, null, $loopItem);
                                        }
                                    }
                                }
                            }
                            
                            $this->procedureGeraNotaFiscalProduto($dadosDevolucao['CDFILIOPELOG'], $proxLancamento["P_PRXLANCTONF"], $produto["CDPRODUTOEST"], $cdProdEmis, $produto["NMPRODUTOEST"], $produto["QTPROGENTAF"], $produto["VRPREUNIAF"], 0, NULL, NULL, NULL, NULL, NULL, 'N', 'S', 0, $produto["CDMARCAOL"]);
                        }
                    } catch(\Exception $e) {
                        $geraNota = false;
                        array_push($this->arrayProdutosErro, array(
                            "CDFILIOPELOG"  => $dadosDevolucao['CDFILIOPELOG'],
                            "NMFILIOPELOG"  => $nmFilial,
                            "CDFILDEST"     => $cdFilDest,
                            "NRLANCTONF"    => $notaFiscal["NRLANCTONF"],
                            "DTENTREGA"     => $notaFiscal["DTENTRSAID"],
                            "DTSAIDA"       => $dadosDevolucao["DATA"],
                            "CDARVPRODOL"   => $produto["CDARVPRODOL"],
                            "NMPRODUTOOL"   => $produto["NMPRODUTOOL"],
                            "CDARVPRODEST"  => $produto["CDARVPRODEST"],
                            "NMPRODUTOEST"  => $produto["NMPRODUTOEST"],
                            "DSERRO"        => $e->getMessage())
                        );
                    }
                }
                
                if($geraNota) {
                    // Realiza o Fechamento da nota
                    $this->lancamentoNotaFiscalService->procedureGeraFechamentoSaida($dadosDevolucao['CDFILIOPELOG'], $proxLancamento["P_PRXLANCTONF"], substr($paramsGeracao["DTPROCESSA"], 0, 10), $proxLancamento["P_PRXLANCESTQ"]);
                    
                    //Verifica se a operação atualiza estoque
                    if($this->operacaoAtualizaEstoque($dadosDevolucao["CDTIPOOPER"]["CDTIPOOPER"], 'S')) {
                        if($paramsGeracao["IDESTONLINE"] == 'S') {
                            // A função atualiza estoque também busca por irregularidades
                            $produtosIrregulares = $this->validacoesEstoqueService->atualizaEstoque($dadosDevolucao['CDFILIOPELOG'], $proxLancamento["P_PRXLANCTONF"]);
                        }
                    }
                    
                    $cdModDocFisc = $this->dfs48000SaidaDoOperadorLogisticoFactory->getCdModDocFisc($dadosDevolucao['CDFILIOPELOG'], $proxLancamento["P_PRXLANCTONF"]);
                    
                    if(empty($cdModDocFisc)) throw new \Exception(self::MSG_ERROS["SEMDOCUFISC"]);
                    
                    $this->verififcaImpostosObrigatorio($dadosDevolucao['CDFILIOPELOG'], $proxLancamento["P_PRXLANCTONF"]);

                    $this->geraRateio($dadosDevolucao['CDFILIOPELOG'], $proxLancamento["P_PRXLANCTONF"], 'S', '', $organizacao);
                    
                    $this->connection->commit();
                        
                    array_push($this->arrayNotasSucesso, array(
                        "CDFILIOPELOG"  => $dadosDevolucao['CDFILIOPELOG'],
                        "NMFILIOPELOG"  => $nmFilial,
                        "CDFILDEST"     => $cdFilDest,
                        "NRLANCTONF"    => isset($notaFiscal["NRLANCTONF"]) ? $notaFiscal["NRLANCTONF"] : "Sem n° de lançamento para esta NF",
                        "DTENTREGA"     => isset($notaFiscal["DTENTRSAID"]) ? $notaFiscal["DTENTRSAID"] : "Sem Dt. Entrega para esta NF",
                        "DTSAIDA"       => $dadosDevolucao["DATA"]
                    ));
                } else {
                    array_push($this->arrayNotasErro, array(
                        "CDFILIOPELOG"  => $dadosDevolucao['CDFILIOPELOG'],
                        "NMFILIOPELOG"  => $nmFilial,
                        "CDFILDEST"     => $cdFilDest,
                        "NRLANCTONF"    => isset($notaFiscal["NRLANCTONF"]) ? $notaFiscal["NRLANCTONF"] : "Sem n° de lançamento para esta NF",
                        "DTENTREGA"     => isset($notaFiscal["DTENTRSAID"]) ? $notaFiscal["DTENTRSAID"] : "Sem Dt. Entrega para esta NF",
                        "DTSAIDA"       => $dadosDevolucao["DATA"],
                        "DSERRO"        => null,
                        "PROD_ERRO"     => $this->arrayProdutosErro)
                    );
                    $this->connection->rollback();
                }
            } catch(\Exception $e) {
                $this->connection->rollback();
                array_push($this->arrayNotasErro, array(
                    "CDFILIOPELOG"  => $dadosDevolucao['CDFILIOPELOG'],
                    "NMFILIOPELOG"  => $nmFilial,
                    "CDFILDEST"     => $cdFilDest,
                    "NRLANCTONF"    => isset($notaFiscal["NRLANCTONF"]) ? $notaFiscal["NRLANCTONF"] : "Sem n° de lançamento para esta NF",
                    "DTENTREGA"     => isset($notaFiscal["DTENTRSAID"]) ? $notaFiscal["DTENTRSAID"] : "Sem Dt. Entrega para esta NF",
                    "DTSAIDA"       => $dadosDevolucao["DATA"],
                    "DSERRO"        => $e->getMessage(),
                    "PROD_ERRO"     => $this->arrayProdutosErro)
                );
            }
        }
        
        if($dadosDevolucao['IDUTRESPEDXOL'] == 'S') {
            self::acmDelReserva($dadosDevolucao['CDFILIOPELOG']);
        }
        
        return array(
            'NOTAS_ERRO'    => $this->arrayNotasErro,
            'NOTAS_SUCESSO' => $this->arrayNotasSucesso
        );
    }
    
    
    /******************************* GERA OCORRENCIA DE ATENDIMENTO *******************************
    *                                           GQL                                               *
    ***********************************************************************************************/
    protected function geraOcorrenciaDeAtendimento($produto, $notaFiscal, $dadosDevolucao, $proxLancamento, $cdOperador, $nmOperador, $cdFilDest, $cdFilEmit, $organizacao) {
        $motivoDev = $this->dfs48000SaidaDoOperadorLogisticoFactory->getMotivoDev($produto["CDMOTIVODEV"]);
        $motivoDev["IDGERAOCORATE"] = 'S';
        
        if($motivoDev["IDGERAOCORATE"] == 'S') {
            $nrMovOcorr  = $this->novoCodigo->getCodeByTableName(self::NOVO_CODIGO_MOVOCORR.$cdFilDest, $organizacao);
            
            $cdFornOpLog = $this->dfs48000SaidaDoOperadorLogisticoFactory->getCdFornOpLog($cdFilEmit)["CDFORNOPELOG"];
        
            $notaOrigem  = $this->dfs48000SaidaDoOperadorLogisticoFactory->getNotaOrigem($notaFiscal["CDFILIAL"], $notaFiscal["NRLANCTONF"]);
            
            $this->dfs48000SaidaDoOperadorLogisticoFactory->insereLote($notaOrigem["CDFILIRECE"], $produto["NRLOTEESTQ"], $produto["NRSUBLOTE"], $produto["CDPRODUTO"]);
            
            $this->dfs48000SaidaDoOperadorLogisticoFactory->insereMovOcorr($cdFilDest, $nrMovOcorr, $notaOrigem["CDFILIRECE"], $cdFornOpLog, $dadosDevolucao["DTSAIDA"], $nmOperador,
                $cdOperador, $produto["CDPRODUTO"], $produto["NRLOTEESTQ"], $produto["NRSUBLOTE"], '', $notaFiscal["CDFILIAL"], $notaFiscal["NRLANCTONF"], $produto["CDMARCA"],
                $dadosDevolucao["DTSAIDA"], 'P', $organizacao);
            
            $dsMovOcorr = $this->strToHex($motivoDev["NMMOTIVODEV"]);
            
            $this->dfs48000SaidaDoOperadorLogisticoFactory->insereDsMovOcorr($cdFilDest, $nrMovOcorr, $dsMovOcorr, $organizacao);
        }
        
        if($motivoDev["IDGERAPEDOL"] == 'S') {
            $nfRelacPexOl = $this->dfs48000SaidaDoOperadorLogisticoFactory->getRelacPexOl($notaFiscal["CDFILIAL"], $notaFiscal["NRLANCTONF"]);
            
            
            if(isset($nfRelacPexOl[0])) {
                $nfRelacPexOl = $nfRelacPexOl[0];
                
                $dtPedPexOl = date('d-m-Y', strtotime("+1 days", strtotime($this->formataDataParaComparacao($motivoDev["DATADIA"]))));
                
                
                $nrPedPexOl = $this->novoCodigo->getCodeByTableName(self::NOVO_CODIGO_NRPEXOL.$nfRelacPexOl["CDFILDEST"].$nfRelacPexOl["CDFILLANC"], $organizacao);
                $this->dfs48000SaidaDoOperadorLogisticoFactory->inserePedExol($nfRelacPexOl["CDFILDEST"], $nfRelacPexOl["CDFILLANC"], $nrPedPexOl, $dtPedPexOl, '1', $nfRelacPexOl["CDEMPRESA"],
                                                                        $nfRelacPexOl["CDFILIOPELOG"], $nfRelacPexOl["IDTIPOEXOL"], $nfRelacPexOl["IDPEDEXOLEXTR"], $notaFiscal["CDFILIAL"].$notaFiscal["NRLANCTONF"], $organizacao);
            
                $lotePedido = '';
                // Lote pedido
                if(empty($motivoDev["CDLOTEENVPEDPRI"])) {
                    $lotePedido = $motivoDev["CDLOTEENVPEDSEG"];
                }
                else if(empty($motivoDev["CDLOTEENVPEDSEG"])) {
                    $lotePedido = $motivoDev["CDLOTEENVPEDPRI"];
                }
                else if(!(empty($motivoDev["CDLOTEENVPEDPRI"])) && !(empty($motivoDev["CDLOTEENVPEDSEG"]))) {
                    $primeiraHora = $this->formataDtHora($motivoDev["DSHORALIMPRI"]);
                    $segundaHora  = $this->formataDtHora($motivoDev["DSHORALIMSEG"]);
                    $horaBanco    = $this->formataDtHora($motivoDev["HORABANCO"]);
                    
                    if(!empty($primeiraHora) && !empty($segundaHora)) {
                        if((strtotime($horaBanco) < strtotime($primeiraHora)) && (strtotime($horaBanco) < strtotime($segundaHora))) {
                            if(strtotime($primeiraHora) < strtotime($segundaHora)) {
                                $lotePedido = $motivoDev["CDLOTEENVPEDPRI"];
                            }
                            else {
                                $lotePedido = $motivoDev["CDLOTEENVPEDSEG"];
                            }
                        }
                        else if((strtotime($horaBanco) < strtotime($primeiraHora)) && (strtotime($horaBanco) > strtotime($segundaHora))) {
                            $lotePedido = $motivoDev["CDLOTEENVPEDPRI"];
                        }
                        else if((strtotime($horaBanco) > strtotime($primeiraHora)) && (strtotime($horaBanco) > strtotime($segundaHora))) {
                             if(strtotime($primeiraHora) < strtotime($segundaHora)) {
                                $lotePedido = $motivoDev["CDLOTEENVPEDPRI"];
                            }
                            else {
                                $lotePedido = $motivoDev["CDLOTEENVPEDSEG"];
                            }
                        }
                    }
                }
            }
            
            $datePedExOl = date('d/m/Y h:i:s', strtotime("+1 days", strtotime(str_replace("/", "-", $motivoDev["DATADIA"]))));
            
            $this->dfs48000SaidaDoOperadorLogisticoFactory->insereItemPedExOl($nfRelacPexOl["CDFILDEST"], $nfRelacPexOl["CDFILLANC"], $nrPedPexOl, $produto["CDPRODUTO"], $produto["QTDEVOLUCAOOL"], 
                                                                    $datePedExOl, $produto["CDMARCA"], $lotePedido, $organizacao);
        }
    }
    
    
    /************************************ ATUALIZA STATUS AF *************************************/
    protected function atualizaStatusAF($produto, $notaFiscal, $dadosDevolucao, $cdFilDest) {
        $itemAutAf    = $this->dfs48000SaidaDoOperadorLogisticoFactory->getItemAutAf($cdFilDest, $notaFiscal["NRLANCTONF"], $produto["CDMARCA"], $produto["CDPRODUTO"])[0];
        
        if($produto["QTDEVOLUCAOOL"] == $itemAutAf["QTEFETENTRAF"]) {
            $idBaixitmAF = '';
            $qtEfetentAF = 0;
        }
        else {
            $idBaixitmAF = '1';
            $qtEfetentAF = $produto["QTITEMPROD"] - $produto["QTDEVOLUCAOOL"];
        }
        
        $this->dfs48000SaidaDoOperadorLogisticoFactory->updateItemAutAf($qtEfetentAF, $idBaixitmAF, $itemAutAf["CDFILDEST"], $itemAutAf["CDFILLANC"], $itemAutAf["NRAUTOFORN"],
                                                                    $itemAutAf["CDMARCA"], $itemAutAf["CDPRODUTO"], $itemAutAf["DTPROGENTAF"]);
                                                                    
        $itemAutAf1 = $this->dfs48000SaidaDoOperadorLogisticoFactory->getItemAutAf2($itemAutAf["CDFILDEST"], $itemAutAf["CDFILLANC"], $itemAutAf["NRAUTOFORN"]);
        if(isset($itemAutAf1[0])) {
            if( $itemAutAf1[0]["NRAUTOFORN"] == "" || $itemAutAf1[0]["NRAUTOFORN"] == "0" ) $this->dfs48000SaidaDoOperadorLogisticoFactory->updateAutoForne('', $itemAutAf["CDFILDEST"], $itemAutAf["CDFILLANC"], $itemAutAf["NRAUTOFORN"]);
            else $this->dfs48000SaidaDoOperadorLogisticoFactory->updateAutoForne('1', $itemAutAf["CDFILDEST"], $itemAutAf["CDFILLANC"], $itemAutAf["NRAUTOFORN"]);
        }
        else {
            $this->dfs48000SaidaDoOperadorLogisticoFactory->updateAutoForne('', $itemAutAf["CDFILDEST"], $itemAutAf["CDFILLANC"], $itemAutAf["NRAUTOFORN"]);
        }
    }
    
    /********************************** INSERE DADOS ADICIONAIS **********************************/
    protected function insereDadosAdicionais($notaFiscal, $proxLancamento, $cdFilDest, $cdFilEmit, $dadosDevolucao, $organizacao, $paramsGeracao) {
        $nrNotaFisc     = $notaFiscal["NRLANCTONF"];
        $dsDadosAdic    = "Filial Origem: $cdFilDest  / Filial Destino:  $cdFilEmit / NF Origem: $nrNotaFisc";
        
        if($notaFiscal['USR_NMROTATRANS'] != NULL) {
            $adqMaxDadosAdic = $this->adqMaxDadosAdic($dadosDevolucao['CDFILIOPELOG'], $notaFiscal['NRLANCTONF']);
            
            if(!empty($adqMaxDadosAdic)) {
                $this->dfs48000SaidaDoOperadorLogisticoFactory->insereDadosAdic($notaFiscal["CDFILDEST"], $proxLancamento["P_PRXLANCTONF"], $adqMaxDadosAdic[0]['NRDADOADIC'], '', $dsDadosAdic, $organizacao);
                
                if($paramsGeracao['IDSISDOS'] == '3') {
                    $adqDadosAdicAF = self::adqDadosAdicAF($dadosDevolucao['CDFILIOPELOG'], $notaFiscal['NRLANCTONF']);
                    
                    if(!empty($adqDadosAdicAF)) {
                        $adqMaxDadosAdic = $this->adqMaxDadosAdic($dadosDevolucao['CDFILIOPELOG'], $notaFiscal['NRLANCTONF']);
                        
                        $dsDadosAdic = "UnOp Emitente: " . $adqDadosAdicAF[0]['CDFILIEMIT'] . " - " . $adqDadosAdicAF[0]['NMFILIEMIT'];
                        
                        $this->dfs48000SaidaDoOperadorLogisticoFactory->insereDadosAdic($notaFiscal["CDFILDEST"], $proxLancamento["P_PRXLANCTONF"], $adqMaxDadosAdic[0]['NRDADOADIC'], '', $dsDadosAdic, $organizacao);
                        
                        $adqMaxDadosAdic = $this->adqMaxDadosAdic($dadosDevolucao['CDFILIOPELOG'], $notaFiscal['NRLANCTONF']);
                        
                        $dsDadosAdic = "UnOp Centralizadora: " . $adqDadosAdicAF[0]['CDFILIFISC'] . " - " . $adqDadosAdicAF[0]['NMFILIFISC'];
                        
                        $this->dfs48000SaidaDoOperadorLogisticoFactory->insereDadosAdic($notaFiscal["CDFILDEST"], $proxLancamento["P_PRXLANCTONF"], $adqMaxDadosAdic[0]['NRDADOADIC'], '', $dsDadosAdic, $organizacao);
                        
                        $adqMaxDadosAdic = $this->adqMaxDadosAdic($dadosDevolucao['CDFILIOPELOG'], $notaFiscal['NRLANCTONF']);
                        
                        $dsDadosAdic = "UnOp Destino: " . $adqDadosAdicAF[0]['CDFILIRECE'] . " - " . $adqDadosAdicAF[0]['NMFILIRECE'];
                        
                        $this->dfs48000SaidaDoOperadorLogisticoFactory->insereDadosAdic($notaFiscal["CDFILDEST"], $proxLancamento["P_PRXLANCTONF"], $adqMaxDadosAdic[0]['NRDADOADIC'], '', $dsDadosAdic, $organizacao);
                        
                        $adqMaxDadosAdic = $this->adqMaxDadosAdic($dadosDevolucao['CDFILIOPELOG'], $notaFiscal['NRLANCTONF']);
                        
                        $dsDadosAdic = "Local de Entrega: " . $adqDadosAdicAF[0]['DSENDEFILISEP'] . ", " . $adqDadosAdicAF[0]['NRENDEFILISEP'] . " - " . $adqDadosAdicAF[0]['DSCOMPENDFIL'] . " - " . $adqDadosAdicAF[0]['NMBAIRFILI'] . " - " . $adqDadosAdicAF[0]['NMMUNICIPIO'] . " - " . $adqDadosAdicAF[0]['SGESTADO'] . " - " . $adqDadosAdicAF[0]['NMPAIS'] . " - " . $adqDadosAdicAF[0]['NRCEPFILI'];
                        
                        $this->dfs48000SaidaDoOperadorLogisticoFactory->insereDadosAdic($notaFiscal["CDFILDEST"], $proxLancamento["P_PRXLANCTONF"], $adqMaxDadosAdic[0]['NRDADOADIC'], '', $dsDadosAdic, $organizacao);
                    }
                }
            }
        }
    }
    
    /******************************* VERIFICA IMPOSTOS OBRIGATORIOS *******************************
    *             Verifica se a nota e seus produtos possuem os impostos obrigatorios             *
    ***********************************************************************************************/
    protected function verififcaImpostosObrigatorio($cdFilial, $nrLanctonf) {
        $notaFiscal   = $this->dfs48000SaidaDoOperadorLogisticoFactory->getDadosNf($cdFilial, $nrLanctonf);
        $impostosObri = $this->dfs48000SaidaDoOperadorLogisticoFactory->getImpostoObrigatorio($notaFiscal["IDENTRSAIDOP"], $notaFiscal["CDTIPOOPER"]);
        $impostosErro = '';
        $produtosComErro = false;
        $notaErro        = false;
        
        foreach ($impostosObri as $impObri) {
            if($impObri["IDPRODNOTA"] == 'P') {
                $impostoProdutos = $this->dfs48000SaidaDoOperadorLogisticoFactory->getImpostoProdutos($cdFilial, $nrLanctonf, $impObri["CDIMPOSTO"]);

                foreach ($impostoProdutos as $impostoProduto) {
                    $produtosComErro = true;
                    array_push($this->arrayImpostosErro, array("NMPRODUTO"  => $impostoProduto["NMPRODUTO"],
                                                         "DSERRO"     => "Produto sem incidência de imposto obrigatório. Imposto: ".$impObri["SGIMPOSTO"]));
                }
            }
            else {
                $impostosCorpo   = $this->dfs48000SaidaDoOperadorLogisticoFactory->getImpostoNota($cdFilial, $nrLanctonf, $impObri["CDIMPOSTO"]);
                if(!isset($impostosCorpo[0])) {
                    $notaErro = true;   
                    $impostosErro.= $impObri["SGIMPOSTO"].', ';
                }
                else if($impostosCorpo[0]["CDFILIAL"] == null) {
                    $notaErro = true;   
                    $impostosErro.= $impObri["SGIMPOSTO"].', ';
                }
            }
        }
        
        if($produtosComErro) throw new \Exception("Notafiscal possui produtos sem incidência de imposto(s) obrigatório(s).");
        $size = strlen($impostosErro);

        if($size != 0) {
            $impostosErro = substr($impostosErro,0, $size-2);
            throw new \Exception("Nota Fiscal sem incidência do(s) imposto(s) obrigatório(s) ".$impostosErro.'.');
        }
        
    }
    
    /******************************* VERIFICA IMPOSTOS OBRIGATORIOS *******************************
    *                                   Gera Rateio Para a nota                                   *
    ***********************************************************************************************/
    protected function geraRateio($cdFilial, $nrLanctonf, $idEntrSaid, $rateioNf, $organizacao, $autMan = 'N', $realizaManut = 'P') {
        if($realizaManut != 'S') {
            if($realizaManut == 'P') {
                $this->dfs48000SaidaDoOperadorLogisticoFactory->deleteRateioNf($cdFilial, $nrLanctonf);
                $notaFiscal = $this->dfs48000SaidaDoOperadorLogisticoFactory->getNotaFiscalRateio($cdFilial, $nrLanctonf);
                //$consContaRateio = $this->dfs48000SaidaDoOperadorLogisticoFactory->getConsContaRateio($idEntrSaid, $notaFiscal["CDTIPOOPER"]);
                
                if($idEntrSaid == 'S') {
                    
                    $tpContaReceber = $this->dfs48000SaidaDoOperadorLogisticoFactory->getTipoContaReceber($cdFilial, $nrLanctonf);
                    $produtosRateio = $this->dfs48000SaidaDoOperadorLogisticoFactory->getItemProdutoRateio($cdFilial, $nrLanctonf);
                    
                    foreach ($produtosRateio as $produto) {
                        $ratCtpr = $this->dfs48000SaidaDoOperadorLogisticoFactory->getRatTpCtpr(isset($tpContaReceber[0]) ? $tpContaReceber[0]["CDTPCTRECEB"] : '');
                        
                        foreach ($ratCtpr as $rat) {
                            $filiRateio = ($rat["CDFILIAL"]) ? $rat["CDFILIAL"] : $notaFiscal["CDFILIEMIT"];
                            $rateioExistente = $this->dfs48000SaidaDoOperadorLogisticoFactory->getRateioNf($cdFilial, $nrLanctonf, $produto["NRSEQITPROD"], $filiRateio, $rat["CDCENTCUST"]);
                            $vrRateNf   = $produto["VRCONTITEM"] * ($produto["VRPERATTPCTR"] / 100);
                            if(isset($rateioExistente[0])) {
                                $this->dfs48000SaidaDoOperadorLogisticoFactory->updateRateioNf($vrRateNf, $rat["VRPERATTPCTR"], $cdFilial, $nrLanctonf, $produto["NRSEQITPROD"], $filiRateio, $rat["CDCENTCUST"]);
                            }
                            else $this->dfs48000SaidaDoOperadorLogisticoFactory->insereRateioNf($vrRateNf, $rat["VRPERATTPCTR"], $cdFilial, $nrLanctonf, $produto["NRSEQITPROD"], $filiRateio, $rat["CDCENTCUST"], $organizacao);
                        }
                    }
                }
                else {
                    $this->insereRateioNf2($cdFilial, $nrLanctonf, $organizacao);
                }
            }
            else {
                
                // Não utilizado por esta rotina
            }
        }
    }
    
    /********************************** VALIDACOES DA NOTAFISCAL **********************************
    * Faz as validações necessarias para a nota                                                   *
    * Caso a nota não seja aprovada em alguma das validações uma exceção é retornada              *
    * e a nota é adicionada ao array de notas com erro $arrayNotasErro                            *
    ***********************************************************************************************/
    protected function validaNotaSaidaOl($notaFiscal, $paramsGeracao, $dtSaida, $cdSerie, $tipoOperEnt) {
        $this->validacoesParamCtrlFisOl($dtSaida, $paramsGeracao, $paramsGeracao["DTPROCESSA"], $paramsGeracao["NRDIASSAIDFIL"], $paramsGeracao["NRDIASSAID"], $paramsGeracao["NRDIASSAIDPD"], $paramsGeracao["NRDIASSAIDPDFIL"]);  
        
        $this->validaSerie($cdSerie);
        $this->validaDataIntegracao($paramsGeracao["CDEMPRESA"], 'S', $tipoOperEnt, $dtSaida);
        $this->validaValorNota($notaFiscal["VRNOTAFISC"]);
        $this->validaDataEmissao($notaFiscal["DTEMISSAO"], $dtSaida);
        $this->validaNrNotaFisc($notaFiscal["NRNOTAFISC"]);
    }
    
    private function validacoesParamCtrlFisOl($dtSaida, $paramsGeracao, $dataProcessamento, $nrDiasSaidFil, $nrDiasSaid, $nrDiasSaidPd, $nrDiasSaidPdFil) {
        $dataProcessamento = $this->formataDataParaComparacao($dataProcessamento);
        $dtSaida = $this->formataDataParaComparacao($dtSaida);
        
        if($paramsGeracao["IDLANCDTMAI"] === 'N' && $paramsGeracao["IDLANCDTMAIFIL"] === 'N' && (strtotime($dtSaida) > strtotime($dataProcessamento)))
            throw new \Exception(self::MSG_ERROS["DAT_MAI"]);
         
        if($paramsGeracao["IDLANCDTMEN"] === 'N' && $paramsGeracao["IDLANCDTMENFIL"] === 'N' && (strtotime($dtSaida) < strtotime($dataProcessamento))) 
            throw new \Exception(self::MSG_ERROS["DAT_MEN"]); 
            
        if($paramsGeracao["IDLANCDTMENFIL"] === 'S' && (strtotime($dtSaida) < strtotime($dataProcessamento))) {
            if(strtotime($dtSaida) < strtotime("-$nrDiasSaidFil days", strtotime($dataProcessamento)))   {
                throw new \Exception(self::MSG_ERROS["DAT_M"]); 
            }
        } else {
            if($paramsGeracao["IDLANCDTMEN"] === 'S' && (strtotime($dtSaida) < strtotime($dataProcessamento))) 
                if(strtotime($dtSaida) < strtotime("-$nrDiasSaid days", strtotime($dataProcessamento)))
                    throw new \Exception(self::MSG_ERROS["DAT_M"]);
        }
        
        if($paramsGeracao["IDLANCDTMAIFIL"] === 'S' && (strtotime($dtSaida) > strtotime($dataProcessamento))) {
            if(strtotime($dtSaida) > strtotime("+$nrDiasSaidPdFil days", strtotime($dataProcessamento))) {
                throw new \Exception(self::MSG_ERROS["DAT_M"]);
            }
        } else {
            if($paramsGeracao["IDLANCDTMAI"] === 'S' && (strtotime($dtSaida) > strtotime($dataProcessamento)))  {
                if(strtotime($dtSaida) > strtotime("+$nrDiasSaidPd days", strtotime($dataProcessamento)))
                    throw new \Exception(self::MSG_ERROS["DAT_M"]);
            }
        }
    }
    
    private function validaSerie($cdSerie) {
        if(!$cdSerie) throw new \Exception(self::MSG_ERROS["SERIE_PAD"]);
    }
    
    private function validaDataIntegracao($cdEmpresa, $idEntrSaidOp, $cdTipoOperac, $dtReferencia) {
        $contopctbl = $this->dfs48000SaidaDoOperadorLogisticoFactory->getContopectbl($cdEmpresa);
        
        if($contopctbl) {
            if($contopctbl[0]) {
                if($idEntrSaidOp == 'E') {
                    if (strtotime($this->formataDataParaComparacao($dtReferencia)) >= $this->getDataTime($contopctbl[0]["DTINIINTCONTDFE"]) && strtotime($this->formataDataParaComparacao($dtReferencia)) <= $this->getDataTime($contopctbl[0]["DTFININTCONTDFE"]))
                        return true;
                    else throw new \Exception(self::MSG_ERROS["LANC_FORTADT"]);
                } else if($idEntrSaidOp == 'S') {
                    if (strtotime($this->formataDataParaComparacao($dtReferencia)) >= strtotime($this->formataDataParaComparacao($contopctbl[0]["DTINIINTCONTDFS"])) && strtotime($this->formataDataParaComparacao($dtReferencia)) <= strtotime($this->formataDataParaComparacao($contopctbl[0]["DTFININTCONTDFS"]))) 
                        return true;
                    else throw new \Exception(self::MSG_ERROS["LANC_FORTADT"]);
                }
            } else return true;
        } else return true;
    }
    
    private function validaValorNota($vrNotaFisc) {
        if(!($vrNotaFisc > 0)) throw new \Exception(self::MSG_ERROS["TOTAL_NF_ZERADO"]);
    }
    
    private function validaDataEmissao($dtEmissao, $dtEntrada) {
        if(is_null($dtEmissao)) throw new \Exception(self::MSG_ERROS["DTEMISSAO_VAZIA"]);
    }
    
    private function validaNrNotaFisc($nrNotaFisc) {
        if(is_null($nrNotaFisc)) throw new \Exception(self::MSG_ERROS["NRNOTAFISC_VAZIO"]);
    }
    
    /**************************** VALIDACOES DOS PRODUTOS DA NOTAFISCAL ****************************
    * Faz as validações necessarias para o produto                                                 *
    * Caso o produto não seja aprovada em alguma das validações uma exceção é retornada            *
    * e o produto é adicionada ao array de produtos com erro $arrayNotasErro                       *
    ************************************************************************************************/
    protected function validaProduto($produto, $notaFiscal, $posicaoEstoqueProd) {
        
        $this->validaProdutoInativo($produto, $notaFiscal["NRLANCTONF"], $notaFiscal["CDFILDEST"]);
        $this->validaValorProduto($produto["VRUNITITEM"]);
        $this->validaQtDevolucao($produto["QTDEVOLUCAOOL"]);
        $this->validaEstoqueDiaProduto($produto, $posicaoEstoqueProd);
        
    }
    
    private function validaProdutoInativo($produto, $nrLanctonf, $cdFilDest) {
        $prodFili = $this->dfs48000SaidaDoOperadorLogisticoFactory->getProdFili($cdFilDest, $produto["CDPRODUTO"]);
        
        if($produto["IDPRODATIVPR"] !== 'S') throw new \Exception(self::MSG_ERROS["PRODUTO_INATIVO"]);
        if(isset($prodFili[0])) {
            if($prodFili[0]["IDPRODATIV"] == 'I') throw new \Exception(self::MSG_ERROS["PRODUTO_INATFIL"]);
        }
        else throw new \Exception(self::MSG_ERROS["PRODUTO_FILIAL"]);
    }
    
    private function validaValorProduto($valorProduto) {
        if(!($valorProduto > 0)) throw new \Exception(self::MSG_ERROS["PRECO_ZERADO"]);
    }
    
    private function validaQtDevolucao($qtDevolucao) {
        if($qtDevolucao == 0) throw new \Exception(self::MSG_ERROS["QTDE_ZERADA"]);
    }
    
    private function validaEstoqueDiaProduto($produto, $posicaoEstoqueProd) {
        //var_dump($posicaoEstoqueProd);die;
        if($posicaoEstoqueProd["QTESTOQDIA"] < ($produto["QTDEVOLUCAOOL"] * $produto["VRFATOCONV"]) 
            && $produto["IDCNTRESTOQ"] != 'N') {
                //var_dump($posicaoEstoqueProd["QTESTOQDIA"]);die;
                throw new \Exception(self::MSG_ERROS["EST_INSUFICIENTE"]);
        }
    }
    
    /***************************************** PROCEDURES *****************************************/
    
    protected function atualizaEstoque($cdFilial, $numLancamento, $dtInicial, $dtFinal, $nrLancto = '') {
        $procedure = new StoredProcedure($this->connection, self::PROCEDURE_ATUALIZA_ESTOQUE);
        $procedure->addParam(new Param('P_FILIMOVI',    Param::PARAM_INPUT, $filial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_FILILANCTO',  Param::PARAM_INPUT, $numLancamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTINICIAL',   Param::PARAM_INPUT, $dtProcessamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTFINAL',     Param::PARAM_INPUT, $numLancEstoque, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTO',    Param::PARAM_INPUT, $nrLancto, Param::PARAM_TYPE_STR));
        
        $procedure->execute();
    
    }
    
    
    /************************************* FUNÇÕES AUXILIARES *************************************/
    
    public function operacaoAtualizaEstoque($cdTipoOper, $idEntrSaidOp = 'S') {
        $infoOperac = $this->dfs48000SaidaDoOperadorLogisticoFactory->getInfoTipoOperac($cdTipoOper, $idEntrSaidOp)[0];
        return !!($infoOperac["IDATUESTTPOP"] == 'S');
    }
    
    protected function getDataTime($data) {
        $data = $this->formataDataParaComparacao($data);
        return strtotime($data);
    }
    
    protected function formataDataParaComparacao($data) {
        $data = substr($data, 0, 10);
        return str_replace("/", "-", $data);
    }
    
    private function maskProduto($cdarvprod, $nrorg) {
        return $this->masks->mask($cdarvprod, preg_replace("/([^.])/", '#', PictureProduto::mascaraProduto($nrorg)));
    }
    
    public function strToHex($string){
        $hex='';
        for ($i=0; $i < strlen($string); $i++){
            $hex .= dechex(ord($string[$i]));
        }
        return $hex;
    }
    
    public function hexToStr($hex){
        $string='';
        for ($i=0; $i < strlen($hex)-1; $i+=2){
            $string .= chr(hexdec($hex[$i].$hex[$i+1]));
        }
        return $string;
    }
    
    public function formataDtHora($dtHora) {
        if(!empty($dtHora)) {
            $dtHora = substr($dtHora, 0, 2).':'.substr($dtHora, 2, 3);
            $date = date('H:i', strtotime($dtHora));
            return $date;
        }
        return null;
    }
    
    /************************************* FUNÇÕES PARA ERROS **************************************     
    * Funções que retornam os erros durante a execução da devolução                                *
    ************************************************************************************************/

    public function getErros() {
        return array_merge($this->getArrayNotasErro(), $this->getArrayProdutosErro());
    }
    
    public function getQtNotasSucesso() {
        return $this->notasSucesso;
    }
    
    public function getArrayNotasErro() {
        return $this->arrayNotasErro;
    } 
    
    public function getArrayProdutosErro() {
        return $this->arrayProdutosErro;
    }
    
    public function getArrayImpostosErro() {
        return $this->arrayImpostosErro;
    }
    
    public function getDtProcessaOpl($row) {
        try {
            $params = array(
                "P_FILOPELOG"  => $row['CDFILIOPELOG'],
                "P_DTENTRSAID" => $row['DTENTRSAID'],
            );
            $query = "SELECT QUERY_DTENTRSAID_DFS48000(:P_FILOPELOG, :P_DTENTRSAID) MESSAGE FROM DUAL";
            $std = $this->connection->prepare($query);
            $std->execute($params);
                
            $resultado = $std->fetchAll();
                
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function getDtProcessaAtual($row) {
        try {
            $params = array(
                "P_CDFILIALOL"  => $row['CDFILIOPELOG']
            );
            $query = "SELECT TO_CHAR(CASE
                                          WHEN PARAMGERAL.IDUTILDTPROC = 'S'
                                          THEN NVL(PARAMFILIAL.DTPROCESSA, TRUNC(SYSDATE))
                                          ELSE TRUNC(SYSDATE)
                                       END, 'DD/MM/YYYY') DTPROCESSA
                          FROM PARAMGERAL, PARAMFILIAL
                         WHERE PARAMFILIAL.CDFILIAL = :P_CDFILIALOL";
            $std = $this->connection->prepare($query);
            $std->execute($params);
                
            $resultado = $std->fetchAll();
                
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function validaItensPedido($row, $dados) {
        try {
            $params = array(
                "P_CDFILIOPELOG"        => $dados['CDFILIOPELOG'],
                "P_CDFILDEST"           => $row['CDFILIAL'],
                "P_CDFILLANC"           => $row['CDFILLANC'],
                "P_NRAUTOFORN"          => $row['NRAUTOFORN'],
                "P_SUBGRUPO"            => $dados['SUBGRUPO'],
                "P_DATAENT"             => $dados['DATAENT'],
                "P_DTENTREGA"           => $dados['DTENTREGA'],
                "P_DTPROC"              => $dados['DTPROC'],
                "P_PEDOLEXTR"           => $dados['PEDOLEXTR'],
                "P_CDLOTENVIOL"         => $dados['CDLOTENVITOL'],
                "P_IDBAIXAOLSAI"        => "T",
                "P_IDUTILCONSSOLIOL"    => $dados['IDUTILCONSSOLIOL'],
                "P_IDUTRESPEDXOL"       => $dados['IDUTRESPEDXOL'],
                "P_DTSAIDANF"           => $dados['DATA'],
                "P_GERA_NF"             => $dados['GERANF'],
                "P_WEBSERVICE"          => $dados['WEBSERVICE']
            );
            $query = "SELECT IDCHECKPRAF, CDFILIAL, CDARVPROD, NMPRODUTO, CDPRODUTO,
                        SGUNIDADE, CDMARCA, CDMARCAOL, CDMARCAAF, NMMARCA, NRAUTOFORN, CDFILLANC,
                        NRLANCTONF, IDBAIXITAFS, CDFILDEST, IDUTILITEM, CDALMOXARIFE, CDLOCALESTOQ, NRLOTEESTQ,
                        NRSUBLOTE, IDUTQTESTRANG, CDCLIENTEEXT, CDARVPRODOL, NMPRODUTOOL, CDARVPRODEST, NMPRODUTOEST,
                        CDFILIOPELOG, NMFILIOPELOG, NMFILIAL, LISTAERRO, DTPROGEMBARQ, DTPROGENTAF, DTPROGENPEOL, QTPROGENTAF,
                        QTSAIDOPLOG, VRPREUNIAF, QTNFRELAFSAI, QTATENDPEDOL, VRPBRUTOPEDOL, VRPLIQUIPEDOL, QTATENDESTRANG,
                        QTESTOQDIA, QTRESERVESTOL, ESTOQ_DISPO, RESERVA_ITEM, QTESTOQDIAEST, VRFATOCONVOL, VRFATOCONV, CDPRODUTOEST
                    FROM TABLE (QUERY_ITEM_DFS48000(:P_CDFILIOPELOG, :P_CDFILDEST, :P_CDFILLANC, :P_NRAUTOFORN, :P_SUBGRUPO,
                                                    :P_DATAENT, :P_DTENTREGA, :P_DTPROC, :P_PEDOLEXTR, :P_CDLOTENVIOL,
                                                    :P_IDBAIXAOLSAI, :P_IDUTILCONSSOLIOL, :P_IDUTRESPEDXOL, :P_DTSAIDANF, :P_GERA_NF,
                                                    :P_WEBSERVICE))";
            $std = $this->connection->prepare($query);
            $std->execute($params);
                
            $resultado = $std->fetchAll();
                
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function getStWebService($row) {
        try {
            $params = array(
                "P_CDFILIOPELOG"  => $row['CDFILIOPELOG']
            );
            $query = "SELECT CDFILIAL
                       FROM PARAMFILIAL
                      WHERE CDFILIAL = :P_CDFILIOPELOG
                        AND CDFILWMSMHA  IS NOT NULL
                        AND CDOPERWMSMHA IS NOT NULL";
            $std = $this->connection->prepare($query);
            $std->execute($params);
                
            $resultado = $std->fetchAll();
                
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqMaxDadosAdic($cdfilial, $nrLanctonf) {
        try {
            $params = array(
                "P_CDFILIAL"   => $cdfilial,
                "P_NRLANCTONF" => $nrLanctonf
            );
            $query = "SELECT NVL(MAX(NRDADOADIC),0) + 1  NRDADOADIC
                          FROM DADOSADIC
                         WHERE CDFILIAL   = :P_CDFILIAL
                           AND NRLANCTONF = :P_NRLANCTONF";
   
            $std = $this->connection->prepare($query);
            $std->execute($params);
                
            $resultado = $std->fetchAll();
                
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqVerifDuplNF($cdfiliopelog, $cdfildest, $cdfillanc, $produto) {
        try {
            $params = array(
                "P_CDFILIAL"    => $cdfiliopelog,
                "P_CDFILDEST"   => $cdfildest,
                "P_CDFILLANC"   => $cdfillanc,
                "P_NRAUTOFORN"  => $produto['NRAUTOFORN'],
                "P_CDMARCA"     => $produto['CDMARCAOL'],
                "P_CDPRODUTO"   => $produto['CDPRODUTOEST'],
                "P_DTPROGENTAF" => substr($produto['DTPROGENTAF'], 0, 10)
            );
            $query = "SELECT NRLANCTONF
                         FROM NFRELACAFSAI
                        WHERE CDFILIAL     = :P_CDFILIAL
                          AND CDFILDEST    = :P_CDFILDEST
                          AND CDFILLANC    = :P_CDFILLANC
                          AND NRAUTOFORN   = :P_NRAUTOFORN
                          AND CDMARCA      = :P_CDMARCA
                          AND CDPRODUTO    = :P_CDPRODUTO
                          AND DTPROGENTAF  = :P_DTPROGENTAF
                       UNION
                       SELECT NRLANCTONF
                         FROM NFRELACPEXOL
                        WHERE CDFILIAL     = :P_CDFILIAL
                          AND CDFILDEST    = :P_CDFILDEST
                          AND CDFILLANC    = :P_CDFILLANC
                          AND NRPEDEXOL    = :P_NRAUTOFORN
                          AND CDMARCA      = :P_CDMARCA
                          AND CDPRODUTO    = :P_CDPRODUTO
                          AND DTPROGENPEOL = :P_DTPROGENTAF";
   
            $std = $this->connection->prepare($query);
            $std->execute($params);
                
            $resultado = $std->fetchAll();
                
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqNFRelacAFSai($cdfiliopelog, $nrlanctonf, $cdfildest, $cdfillanc, $produto) {
        try {
            $params = array(
                "P_CDFILIAL"    => $cdfiliopelog,
                "P_NRLANCTONF"  => $nrlanctonf,
                "P_CDFILDEST"   => $cdfildest,
                "P_CDFILLANC"   => $cdfillanc,
                "P_NRAUTOFORN"  => $produto['NRAUTOFORN'],
                "P_CDMARCA"     => $produto['CDMARCAOL'],
                "P_CDPRODUTO"   => $produto['CDPRODUTO'],
                "P_DTPROGENTAF" => substr($produto['DTPROGENTAF'], 0, 10)
            );
            $query = "SELECT NRLANCTONF
                       FROM NFRELACAFSAI
                       WHERE CDFILIAL    = :P_CDFILIAL    AND
                             NRLANCTONF  = :P_NRLANCTONF  AND
                             CDFILDEST   = :P_CDFILDEST   AND
                             CDFILLANC   = :P_CDFILLANC   AND
                             NRAUTOFORN  = :P_NRAUTOFORN  AND
                             CDMARCA     = :P_CDMARCA     AND
                             CDPRODUTO   = :P_CDPRODUTO   AND
                             DTPROGENTAF = :P_DTPROGENTAF";
   
            $std = $this->connection->prepare($query);
            $std->execute($params);
                
            $resultado = $std->fetchAll();
                
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqNFRELACPEXOL($cdfiliopelog, $nrlanctonf, $cdfildest, $cdfillanc, $produto, $data = null) {
        try {
            $params = array(
                "P_CDFILIAL"    => $cdfiliopelog,
                "P_NRLANCTONF"  => $nrlanctonf,
                "P_CDFILDEST"   => $cdfildest,
                "P_CDFILLANC"   => $cdfillanc,
                "P_NRPEDEXOL"   => $produto['NRAUTOFORN'],
                "P_CDMARCA"     => $produto['CDMARCAOL'],
                "P_CDPRODUTO"   => $produto['CDPRODUTOEST'],
                "P_DTPROGENPEOL"=> $data == null ? substr($produto['DTPROGENPEOL'], 0, 10) : substr($data, 0, 10)
            );
            $query = "SELECT NRLANCTONF
                      FROM NFRELACPEXOL
                      WHERE CDFILIAL     = :P_CDFILIAL    AND
                            NRLANCTONF   = :P_NRLANCTONF  AND
                            CDFILDEST    = :P_CDFILDEST   AND
                            CDFILLANC    = :P_CDFILLANC   AND
                            NRPEDEXOL    = :P_NRPEDEXOL   AND
                            CDMARCA      = :P_CDMARCA     AND
                            CDPRODUTO    = :P_CDPRODUTO   AND
                            DTPROGENPEOL = :P_DTPROGENPEOL";
   
            $std = $this->connection->prepare($query);
            $std->execute($params);
                
            $resultado = $std->fetchAll();
                
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqExistItem($cdfildest, $cdfillanc, $produto) {
        try {
            $params = array(
                "P_CDFILDEST"   => $cdfildest,
                "P_CDFILLANC"   => $cdfillanc,
                "P_NRPEDEXOL"   => $produto['NRAUTOFORN'],
                "P_CDMARCA"     => $produto['CDMARCAOL'],
                "P_CDPRODUTO"   => $produto['CDPRODUTO'],
                "P_DTPROGEMBARQ"=> substr($produto['DTPROGEMBARQ'], 0, 10)
            );
            $query = "SELECT
                        DTPROGENPEOL
                     FROM
                        ITEMPEDEXOL
                     WHERE
                        CDFILDEST = :P_CDFILDEST AND
                        CDFILLANC = :P_CDFILLANC AND
                        NRPEDEXOL = :P_NRPEDEXOL AND
                        CDMARCA = :P_CDMARCA AND
                        CDPRODUTO = :P_CDPRODUTO AND
                        DTPROGEMBARQ = :P_DTPROGEMBARQ";
   
            $std = $this->connection->prepare($query);
            $std->execute($params);
                
            $resultado = $std->fetchAll();
                
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function acmInsNfRelacAfSai($cdfilial, $cdfillanc, $nrlanctonf, $cdfildest, $produto) {
        $sql = "INSERT INTO NFRELACAFSAI
                (CDFILIAL, CDFILLANC, NRLANCTONF, CDFILDEST,
                 NRAUTOFORN, CDPRODUTO, CDMARCA,
                 DTPROGENTAF, QTNFRELAFSAI, VRNFRELAFSAI)
                VALUES
                (:CDFILIAL, :CDFILLANC, :NRLANCTONF, :CDFILDEST,
                 :NRAUTOFORN, :CDPRODUTO, :CDMARCA, :DTPROGENTAF,
                 :QTNFRELAFSAI, :VRNFRELAFSAI)";
                  
        $params = array(
            "CDFILIAL"      => $cdfilial,
            "CDFILLANC"     => $cdfillanc,
            "NRLANCTONF"    => $nrlanctonf,
            "CDFILDEST"     => $cdfildest,
            "NRAUTOFORN"    => $produto['NRAUTOFORN'],
            "CDPRODUTO"     => $produto['CDPRODUTO'],
            "CDMARCA"       => $produto['CDMARCAOL'],
            "DTPROGENTAF"   => substr($produto['DTPROGENTAF'], 0, 10),
            "QTNFRELAFSAI"  => $produto['QTPROGENTAF'],
            "VRNFRELAFSAI"  => $produto['VRPREUNIAF']
        );
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    public function acmInsNFRELACPEXOL($cdfilial, $cdfillanc, $nrlanctonf, $cdfildest, $produto, $data, $loopItem) {
        $sql = "INSERT INTO NFRELACPEXOL
                 (CDFILIAL, CDFILLANC, NRLANCTONF, CDFILDEST,
                  NRPEDEXOL, CDPRODUTO, CDMARCA,
                  DTPROGENPEOL, QTNFRELPEXOL, VRNFRELPEXOL, NRSEQITPEOL)
                 VALUES
                 (:CDFILIAL, :CDFILLANC, :NRLANCTONF, :CDFILDEST,
                  :NRPEDEXOL, :CDPRODUTO, :CDMARCA, :DTPROGENPEOL,
                  :QTNFRELPEXOL, :VRNFRELPEXOL, :NRSEQITPEOL)";
                  
        $params = array(
            "CDFILIAL"      => $cdfilial,
            "CDFILLANC"     => $cdfillanc,
            "NRLANCTONF"    => $nrlanctonf,
            "CDFILDEST"     => $cdfildest,
            "NRPEDEXOL"     => $produto['NRAUTOFORN'],
            "CDPRODUTO"     => $produto['CDPRODUTO'],
            "CDMARCA"       => $produto['CDMARCAOL'],
            "DTPROGENPEOL"  => $data == null ? substr($produto['DTPROGENTAF'], 0, 10) : substr($data, 0, 10),
            "QTNFRELPEXOL"  => $produto['QTPROGENTAF'],
            "VRNFRELPEXOL"  => $produto['VRPREUNIAF'],
            "NRSEQITPEOL"   => $loopItem
        );
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    public function acmDelReserva($cdfiliopelog) {
        $sql = "DELETE
                  FROM RESERVAPEDEXOL R
                 WHERE R.CDFILRESOL = :P_FILOPELOG
                   AND EXISTS (SELECT 1
                                 FROM ITEMPEDEXOL I
                                WHERE I.CDFILDEST    = R.CDFILDEST
                                  AND I.CDFILLANC    = R.CDFILLANC
                                  AND I.NRPEDEXOL    = R.NRPEDEXOL
                                  AND I.CDPRODUTO    = R.CDPRODUTO
                                  AND I.CDMARCA      = R.CDMARCA
                                  AND I.DTPROGENPEOL = R.DTPROGENPEOL
                                  AND NVL(I.IDBAIXAOLSAI, 'O') IN ('1', '2', '3'))";
                  
        $params = array(
            "P_FILOPELOG"   => $cdfiliopelog
        );
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    public function adqDadosAdicAF($cdfiliopelog, $nrlanctonf) {
        try {
            $params = array(
                "P_CDFILIAL"    => $cdfiliopelog,
                "P_NRLANCTONF"  => $nrlanctonf
            );
            $query = "SELECT N.CDFILIEMIT, F.NMFILIAL AS NMFILIEMIT, D.CDFILIAL AS CDFILIRECE, D.NMFILIAL AS NMFILIRECE,
                        F.CDFILIFISC, C.NMFILIAL AS NMFILIFISC,
                        E.DSENDEFILI AS DSENDEFILISEP, E.NRENDEFILISEP, E.DSCOMPENDFIL, E.NMBAIRFILI,
                        M.NMMUNICIPIO, M.SGESTADO, P.NMPAIS, E.NRCEPFILI
                  FROM NOTAFISCAL N, FILIAL F, FILIAL C, FILIAL D, ENDEFILI E, PAIS P, MUNICIPIO M
                 WHERE N.CDFILIAL = :P_CDFILIAL
                   AND N.NRLANCTONF = :P_NRLANCTONF
                   AND F.CDFILIAL = N.CDFILIEMIT
                   AND F.CDFILIFISC = C.CDFILIAL(+)
                   AND D.CDFILIAL = N.CDFILIRECE
                   AND E.CDFILIAL = N.CDFILIRECE
                   AND E.IDTPENDEFILI = 'P'
                   AND P.CDPAIS = E.CDPAIS
                   AND M.CDPAIS = E.CDPAIS
                   AND M.SGESTADO = E.SGESTADO
                   AND M.CDMUNICIPIO = E.CDMUNICIPIO
                   AND NOT EXISTS (SELECT CDFILIAL
                                     FROM ENDEFILI
                                    WHERE ENDEFILI.CDFILIAL = N.CDFILIRECE
                                      AND ENDEFILI.IDTPENDEFILI = 'T')
                UNION
                SELECT N.CDFILIEMIT, F.NMFILIAL AS NMFILIEMIT, D.CDFILIAL AS CDFILIRECE, D.NMFILIAL AS NMFILIRECE,
                        F.CDFILIFISC, C.NMFILIAL AS NMFILIFISC,
                        E.DSENDEFILI AS DSENDEFILISEP, E.NRENDEFILISEP, E.DSCOMPENDFIL, E.NMBAIRFILI,
                        M.NMMUNICIPIO, M.SGESTADO, P.NMPAIS, E.NRCEPFILI
                  FROM NOTAFISCAL N, FILIAL F, FILIAL C, FILIAL D, ENDEFILI E, PAIS P, MUNICIPIO M
                 WHERE N.CDFILIAL = :P_CDFILIAL
                   AND N.NRLANCTONF = :P_NRLANCTONF
                   AND F.CDFILIAL = N.CDFILIEMIT
                   AND F.CDFILIFISC = C.CDFILIAL(+)
                   AND D.CDFILIAL = N.CDFILIRECE
                   AND E.CDFILIAL = N.CDFILIRECE
                   AND E.IDTPENDEFILI = 'T'
                   AND P.CDPAIS = E.CDPAIS
                   AND M.CDPAIS = E.CDPAIS
                   AND M.SGESTADO = E.SGESTADO
                   AND M.CDMUNICIPIO = E.CDMUNICIPIO";
   
            $std = $this->connection->prepare($query);
            $std->execute($params);
                
            $resultado = $std->fetchAll();
                
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function procedureIncluiReservaOL($row) {
        $procedure = new StoredProcedure($this->connection, 'INCLUI_RESERVA_OL');
        $procedure->addParam(new Param('P_FILOPELOG',       Param::PARAM_INPUT,     $row['P_FILOPELOG'],    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTENTREGA',       Param::PARAM_INPUT,     $row['P_DTENTREGA'],    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DATAENT',         Param::PARAM_INPUT,     $row['P_DATAENT'],      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_ROTA',            Param::PARAM_INPUT,     $row['P_ROTA'],         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_PEDOLEXTR',       Param::PARAM_INPUT,     $row['P_PEDOLEXTR'],    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDLOTENVIOL',     Param::PARAM_INPUT,     $row['P_CDLOTENVIOL'],  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_SUBGRUPO',        Param::PARAM_INPUT,     $row['P_SUBGRUPO'],     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_STATUS',          Param::PARAM_INPUT,     $row['P_STATUS'],       Param::PARAM_TYPE_STR));

        $procedure->addParam(new Param('P_ROWSAFFECT',      Param::PARAM_OUTPUT,    NULL,                   Param::PARAM_TYPE_STR, 10));
        
        $procedure->addParam(new Param('P_VERIFICACNPJ',    Param::PARAM_INPUT,     $row['P_VERIFICACNPJ'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRINSJUREMPR',    Param::PARAM_INPUT,     $row['P_NRSJUREMPR'],   Param::PARAM_TYPE_STR));
        
        return $procedure->execute();
    }
}
