<?php

require_once 'base/GepSituacaoEntityBase.php';

/**
 * GepSituacaoEntity.
 *
 * @package Entity
 * @author  tarcisiopereira <username@example.com>
 */
class GepSituacaoEntity extends GepSituacaoEntityBase
{


    const RESULTADO_SEM_REGISTRO = 10;
    const SEM_PARCEIROS = 20;


    /**
    * Descri?: Gera o novo c?o de Situacao por Tipo de Fluxo.
    *
    * @param integer $nrorg N?o da Organiza?.
    *
    * @return $codigo integer
    */
    public static function retornaProxNumeroSituacao($nrorg){
        $codigo = Novocodigo::geraCodigo($nrorg, 'GEP_SITUACAO', 10);
        return $codigo;
    }


    /**    ok
     * Retorna os dados da situacao informada pelo operador.
     *
     * @param integer $nrorg       Numero da organizacao.
     * @param integer $nrsituacao  Numero da situacao.
     * @param integer $nrtipofluxo Numero do tipo de fluxo.
     *
     * @return Ambigous <mixed, NULL>|NULL
     */
    public static function retornaSituacao($nrorg, $nrsituacao, $nrtipofluxo){
        $dados = GepSituacao::buscaGepSituacao(
            $nrorg,
            $nrsituacao,
            $nrtipofluxo
        );

        if ($dados instanceof GepSituacaoEntity) {
            return $dados;
        }

        throw new Exception('GepSituacaoEntity: Nao ha registro para esta situacao.', self::RESULTADO_SEM_REGISTRO);
    }


    /**
     * Retorna os poss?is parceiros respons?is pela situa?.
     *
     * @param integer $nrorg       N?o da Organiza?.
     * @param integer $situacao    N?o da situa?.
     * @param integer $parceiro    N?o do Parceiro.
     * @param integer $solicitacao N?o da Solicita?.
     *
     * @return array
     */
    public static function retornaParcRespTramitacao($nrorg, $situacao, $parceiro, $solicitacao){
        $dados = GepSituacao::retornaParcRespTramitacao($nrorg, $situacao, $parceiro, $solicitacao);

        if (count($dados) > 0) {
            return $dados;
        }

        throw new Exception('GepSituacaoEntity: Nao ha parceiros para esta situacao!', self::SEM_PARCEIROS);
    }


    /**
     * Retorna a quantidade de situacoes de determinado tipo de fluxo.
     *
     * @param integer $nrorg       Numero da organizacao.
     * @param integer $nrtipofluxo Numero do tipo de fluxo.
     *
     * @throws Exception           Sem registro de consulta.
     * @return Ambigous <multitype:, NULL, TkModelEntity>
     */
    public static function retornaSituacaoPorTipoFluxo($nrorg, $nrtipofluxo){
        $consulta = GepSituacao::consultaSituacaoPorTipoFluxo($nrorg, $nrtipofluxo);
        if (count($consulta[0])>0) {
            return $consulta;
        }

        throw new Exception(
            'GepSituacaoEntity: Nao ha situacao para este tipo de fluxo!',
            self::RESULTADO_SEM_REGISTRO
        );
    }


    /**       ok
     * Retorna o SQL para searchs de operadores respons?veis em uma situacao.
     *
     * @param integer $nrorg       Numero da Organizacao.
     * @param integer $situacao	   Numero da situacao atual.
     * @param integer $parceiro    Codigo do Parceiro.
     * @param integer $solicitacao Numero da Solicitacao.
     *
     * @return string
     */
    public function retornaSqlOperRespSituacao($nrorg, $situacao, $parceiro, $solicitacao){
        $sql = GepSituacao::retornaSqlParcRespTramitacao($nrorg, $situacao, $parceiro, $solicitacao);
        return $sql;
    }


    /**           ok
     * Retorna o sql com os perfis de acesso respons?veis por uma determiada situacao.
     *
     * @param integer $nrorg    Numero da Organizacao.
     * @param integer $situacao Numero da Situacao.
     *
     * @return string
     */
    public function retornaSqlPerfilAcessoSituacao($nrorg, $situacao){
        $perfil = GepSituacao::sqlPerfilAcessoSituacao($nrorg, $situacao);
        return $perfil;
    }


}